/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012,2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.configure.ui.internal.DeploySamples"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.ActionRegistry"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.ui.SimpleToolbar"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("dojox.form.Uploader"); //$NON-NLS-1$
dojo.require("dojox.form.uploader.FileList"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog"); //$NON-NLS-1$
dojo.require('com.ibm.relm.configure.ui.internal.ArtifactElementConfirmationDialog'); //$NON-NLS-1$


dojo.requireLocalization("com.ibm.relm.configure.ui.internal", "ConfigurePageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
var HTML = jazz.util.html;

dojo.declare(
	"com.ibm.relm.configure.ui.internal.DeploySamples", //$NON-NLS-1$
	[dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.configure", "ui/internal/templates/DeploySamples.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.configure.ui.internal", "ConfigurePageMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		this._table = new jazz.ui.StyledTable({
			columns: [
	          	{label: this._messages.headName, width: "30%", sortable: true},  //$NON-NLS-1$
	          	{label: this._messages.headDescription, width: "55%", sortable: false},  //$NON-NLS-1$
	          	{label: this._messages.headActions, width: "15%", className: "action-column", sortable: false} //$NON-NLS-1$ //$NON-NLS-2$
			], 
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS
		});
		this.content.appendChild(this._table.domNode);
		
		this.loadContentPackages();
	},
	
	loadContentPackages: function(){
		var self = this;
		jazz.client.xhrGet({
			url: net.jazz.ajax._contextRoot + "/contentPackage", //$NON-NLS-1$
			headers: {
				"X-ibm-rmps-internal": "true" //$NON-NLS-1$ //$NON-NLS-2$
			},
			handleAs: "json",  //$NON-NLS-1$
			load: function(data){
				self._renderResults(data);
			}
		});
	},
	
	_renderResults: function(results) {
		this._table.clearBody();
		if (results && results.length > 0) {
			for (var i = 0; i < results.length; i++) {
				this._table.tbody.appendChild(this._createQueryRow(results[i]));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	_createQueryRow: function(packageInfo) {
		//Backporting Workitem 112133: Prevent XSS while loading the contents.
		packageInfo.label = HTML.escape(packageInfo.label);
		packageInfo.folder = HTML.escape(packageInfo.folder);
		packageInfo.description = HTML.escape(packageInfo.description);
		
		
		// create row
		var row = dojo.create("tr"); //$NON-NLS-1$
		// create title column
		var td = dojo.create("td", {innerHTML: packageInfo.label, className: "nameColumn"}, row); //$NON-NLS-1$ //$NON-NLS-2$

		// create description column
		td = dojo.create("td", {innerHTML: packageInfo.description || "", className: "descriptionColumn"}, row); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		// create actions column
		td = dojo.create("td", {className: "actionsColumn"}, row); //$NON-NLS-1$ //$NON-NLS-2$
		var button = dojo.create("button", {className: "actionButton", innerHTML: this._messages.deploy}, td); //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(button, "onclick", dojo.hitch(this, "deployPackage", {button: button, packageInfo:packageInfo})); //$NON-NLS-1$ //$NON-NLS-2$

		return row;
	},

	_createNoneMessage: function() {
		var row = dojo.create("tr"); //$NON-NLS-1$
		dojo.create("td", {innerHTML: HTML.escape(this._messages.noContentsFound)}, row); //$NON-NLS-1$
		return row;
	},
	
	deployPackageMain: function(args){
		var button = args.button;
		button.disabled = true;
		var content = {
			folder: args.packageInfo.folder,
			projectAreaUuid: args.projectArea.itemId				
		};
		if(args.sameIdentifierOptions){
			content.sameIdentifierOptions = args.sameIdentifierOptions; 
		}
		var self = this;
		JazzUIWorkbench.showMessage(self._messages.status_deploying);
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + "/contentPackage",//$NON-NLS-1$
			content: content,
			handleAs: "json",//$NON-NLS-1$
			headers: {
				"X-jazz-project-id": this.itemId, //$NON-NLS-1$
				"X-ibm-rmps-internal": "true" //$NON-NLS-1$ //$NON-NLS-2$
			},
			load: function(response, ioargs){
				JazzUIWorkbench.hideMessageArea();
				if(ioargs.xhr.status == 202 && response.dupIdList){
					new com.ibm.relm.configure.ui.internal.ArtifactElementConfirmationDialog({
						dupIdList: response.dupIdList,
						onOk: function(option){
							var newArgs = dojo.mixin({sameIdentifierOptions: option}, args); //$NON-NLS-1$
							self.deployPackageMain(newArgs);
						}
					});
					return;
				}
				self.messageArea.showMessage({summary: self._messages.status_deployed, closable: true});
			},
			error: function(response, ioargs){
				JazzUIWorkbench.hideMessageArea();
				console.error(response.message);
		    	var message = com.ibm.relm.explorer.web.ui.internal.util.getXhrErrorMessage(response, ioargs);
		    	self.messageArea.showMessage(message);		
			},
			handle: function(){
				button.disabled = ""; //$NON-NLS-1$
			}
		});
	},
	
	deployPackage: function(args) {
		var self = this;
		var params = {
			projects: true,
			teams: false,
			allowsMultiple: false,
//			selection: self._folder.processAreaUUID,
//			filterProjectArea: self._folder.projectAreaUUID,
			onOk: function(selectedItem, action) {
				args.projectArea = selectedItem;
				self.deployPackageMain(args);
			},
		};
		new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);
		
	}
	
});

})();