/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.internal.QueryUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.CascadingComboBox"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-2$ //$NON-NLS-1$

(function() {

var queryUtils = com.ibm.relm.common.web.ui.internal.QueryUtils;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;
var util = com.ibm.relm.explorer.web.ui.internal.util;

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent", [dijit._Widget, dijit._Templated] , { //$NON-NLS-1$
//
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/templates/AddHoverPropertyContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
	widgetsInTemplate: true,
	
	messages: null,
	
	// properties set by caller
	queries: null,
	diagramEditor: null,
	nodeProperties: null,
	resourceUri: null,
	
	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);

		// add "query" field
		var self = this;
		dojo.forEach(this.queries || [], function(queryTemplate) {
			var optionChild = dojo.doc.createElement("option"); //$NON-NLS-1$
			optionChild.innerHTML =  util.escapeHtml(queryTemplate.id);
			optionChild.value= queryTemplate.id;
			self.queryField.appendChild(optionChild);
		});

		// initialize "name" field
		this.valueTypeChanged();
	},
	
	getValues: function(){
		var query = "";//$NON-NLS-1$
		if(this.typeNode.checked){
			query = "__node_property__";//$NON-NLS-1$
		}else if(this.typeQuery.checked && this.queryField.value != this.messages.DefaultQuery){
			query = this.queryField.value;
		}

		return {
			label: this.labelField.value,
			layout: this.layoutField.value,
			query: query,
			name: this.nameField.getValue() || ""//$NON-NLS-1$
		};
	},
	
	valueTypeChanged: function(){
		if(this.typeQuery.checked){
			this.queryChanged();
		}else{
			if(this.typeNode.checked){
				var store = null;
				if(this.nodeProperties){
					store = new dojo.store.Memory({idProperty: "name", data: this.nodeProperties});//$NON-NLS-1$
				}
				this.nameField.set("store", store || new dojo.store.Memory());//$NON-NLS-1$
			}else{ // typeProperty
				queryUtils.getPredicates(this.resourceUri).then(dojo.hitch(this, function(data){
					data.sort(function(a, b){
						return a.name.toLowerCase() > b.name.toLowerCase() ? 1 : -1;
					});
					var store = new dojo.store.Memory({data: data});
					this.nameField.set("labelAttr", "name"); //$NON-NLS-1$//$NON-NLS-2$
					this.nameField.set("searchAttr", "value"); //$NON-NLS-1$//$NON-NLS-2$
					this.nameField.set("store", store); //$NON-NLS-1$
				}));
			}
		}
		this.nameField.set("labelAttr", "name"); //$NON-NLS-1$//$NON-NLS-2$
		this.nameField.set("searchAttr", "value"); //$NON-NLS-1$//$NON-NLS-2$
		this.valueTypeQueryRow.style.display = this.typeQuery.checked ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	queryChanged: function(){
		var store = null;
		if(this.diagramEditor){
			var queryVariables = editorUtils.getQueryVariablesForTarget(this.diagramEditor, this.queryField.value);
			store = new dojo.store.Memory({data: queryVariables});
		}else{
			store = new dojo.store.Memory();
		}
		this.nameField.set("store", store);//$NON-NLS-1$
	}
});

com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent.openDialog = function(resource, props, callback2){

	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent(props);

	// show dialog
	new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: contentWidget.messages.AddHoverProperty,
		onOpen: function(){
			dijit.focus(contentWidget.labelField);
		},
		validate: function(values, callback){
			var label = values["label"]; //$NON-NLS-1$
			if(label && label.length>0){
				var prop = null;
				var hoverProps = resource.hoverProps;
				if(hoverProps) {
					for(var i = 0; i < hoverProps.length; i++) {
						if(hoverProps[i].label == label) {
							prop = hoverProps[i];
							break;
						}
					}
				}
				if(prop) {
					alert(contentWidget.messages.errPropertyNameExists);
				} else {
					callback();
				}
			}else{
				alert(contentWidget.messages.errPropertyNameEmpty);
			}
		},
		onOK: function(values){
			if(resource.hoverProps == null) {
				resource.hoverProps = [];
			}

			resource.hoverProps.push({label: values["label"] , //$NON-NLS-1$
							layout: values["layout"] , //$NON-NLS-1$
							baseQueryId: values["query"] , //$NON-NLS-1$
							name: values["name"] }); //$NON-NLS-1$
			if(callback2) {
				callback2();
			}
		}
	});
};

})();