/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.gfx"); //$NON-NLS-1$

dojo.require("com.ibm.relm.common.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.Section"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ShapeDictionary"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
var util = com.ibm.relm.common.web.ui.internal.util;
var shapesTypes = null;
var attrUIExtensions = [];
var fontAttributes = {family: "", size: "", weight: "", style: "", variant: ""}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
var Dictionary = com.ibm.relm.explorer.web.ui.internal.view.common.ShapeDictionary;

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog", [dijit._Widget, dijit._Templated] , { //$NON-NLS-1$
	shapeDef: null, // target shape Definition

	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/templates/ShapeEditDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	messages:null,
	
	_attrsRow: null,
	_advancedShown: false,
	
	postMixInProperties: function(){
		this._shapeAttrLabel = dijit.getUniqueId("shapeAttrLabel"); //$NON-NLS-1$
		this._shapeTypeFieldId = dijit.getUniqueId("shapeTypeField"); //$NON-NLS-1$
		this._shapeAdvancedFieldId = dijit.getUniqueId("shapeAdvancedField"); //$NON-NLS-1$
		var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$;
		this.messages = messages;
		if(!shapesTypes){
			shapesTypes = [{type:"Rect", label:messages.shapeRectangle, additionalAttrs: ["fill","stroke"]},//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
				{type:"Ellipse", label:messages.shapeEllipse, additionalAttrs: ["fill","stroke"]},//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
				{type:"Circle", label:messages.shapeCircle, additionalAttrs: ["fill","stroke"]},//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
				{type:"Image", label:messages.shapeImage},//$NON-NLS-1$
				{type:"Text", label:messages.shapeText, additionalAttrs: ["font","fill","stroke"]},//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
				{type:"MLText", label:messages.shapeMultiLinePath, attrArias: "Text", additionalAttrs: ["width","wrap","wordWrap"]},//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$
				{type:"TextPath", label:messages.shapeTextPath, additionalAttrs: ["font","fill","stroke","path"]}, //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$
				{type:"Line", label:messages.shapeLine, additionalAttrs: ["fill","stroke"]},//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
				{type:"Polyline", label:messages.shapePolyline, additionalAttrs: ["fill","stroke"]},//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
				{type:"Path", label:messages.shapePath, additionalAttrs: ["fill","stroke"]}];//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		}
	},
	
	postCreate: function(){
		this.inherited(arguments);
		dojo.forEach(shapesTypes, function(shapeType){
			var options = this.shapeTypeField.options;
			options[options.length] = new Option(shapeType.label, shapeType.type);
		}, this);
		if(this.shapeDef && this.shapeDef.type){
			this.shapeTypeField.value = this._getShapeType(this.shapeDef.type);
		}

		this._createUIExtensions();
		this._shapeTypeChanged(this.shapeDef);
	},
	
	onShapeTypeChanged: function(e){
		this._shapeTypeChanged();
	},
	
	onShapeAdvancedChanged: function(e){
		try {
			var newShape = dojo.fromJson( dojo.trim(this.shapeAdvancedField.value));
			this._shapeTypeChanged(newShape);
		} catch (e) {
			// reset advanced field to pre-edit
			alert(this.messages.invalidJsonFormat);
			this._shapeTypeChanged(this.getValues().value);
		}
	},
	
	toggleAdvanced: function(ev){
		var shown = this._advancedShown;
		shown = !shown;
		this._toggleShowAdvanced(!shown);
		dojo.stopEvent(ev);
	},
	
	_toggleShowAdvanced: function(showAdvanced){
		this.shapeAdvancedTable.style.display = showAdvanced ? "none" : ""; //$NON-NLS-1$ //$NON-NLS-2$
		showAdvanced ? dojo.replaceClass(this.advancedLink, "showAdvanced", "hideAdvanced") : dojo.replaceClass(this.advancedLink, "hideAdvanced", "showAdvanced"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		this.advancedLinkMessage.innerHTML = showAdvanced ? this.messages.shapeShowAdvancedLink : this.messages.shapeHideAdvancedLink;
		this._advancedShown = !showAdvanced;
	},

	_getDefaultShape: function(shapeType) {
		// In case there is arias, attributes are retrieved from the arias shape
		var type = shapeType;
		dojo.some(shapesTypes, function(s) {
			if (s.type == shapeType) {
				if (s.attrArias != undefined) {
					type = s.attrArias;
				}
				return true;
			}
		});

		var shape = dojox.gfx.getDefault(type);
		if(!shape){
			return null;
		}
		
		if (type != shapeType) {
			shape.type = shapeType.toLowerCase();
		}
		
		dojo.forEach(shapesTypes, function(s){
			if(s.type == type || s.type == shapeType){
				dojo.forEach(s.additionalAttrs, function(a){
					if(shape[a]) return;
					shape[a] = a == "font" ? dojo.clone(fontAttributes) : ""; //$NON-NLS-1$ //$NON-NLS-2$
				});
			}
		});
	
		return shape;
	},
	
	_shapeTypeChanged: function(initialValue){
		initialValue = initialValue ? dojo.clone(initialValue) : {};
		var advancedValue = dojo.clone(initialValue);
		var currentValue = this.getValues().value || {};
		dojo.forEach(this._attrsRow, function(r){
			this.shapeAttrsTable.removeChild(r.domNode);
		}, this);
		this._attrsRow = [];
		
		var shapeType = this.shapeTypeField.value;
		var shape = this._getDefaultShape(shapeType);
		if (!shape) {
			return
		}
		
		var attrSections = {normalType: {label: this.messages.shapeAttrsLabel, id: dijit.getUniqueId("shapeAttrLabelId"), values:{}}}; //$NON-NLS-1$
		for(var attr in shape){
			switch (attr) {
			case "type": //$NON-NLS-1$
				attrSections.normalType.shapeType = shape.type;
				break;
			case "font": //$NON-NLS-1$
				var section = null;
				if (!attrSections[attr]) {
					section = {};
					section.shapeType = attr;
					section.label = this.messages.shapeFontAttrsLabel;
					section.id = dijit.getUniqueId("shapeFontAttrLabelId");	//$NON-NLS-1$
					attrSections[attr] = section;
				} else {
					section = attrSections[attr];
				}
				section.values = shape[attr];
				var initVal = initialValue[attr] || {};
				for(var a in initVal) {
					section.values[a] = initVal[a];
				}
				break;
			default:
				attrSections.normalType.values[attr] = initialValue[attr] || currentValue[attr] || ""; //$NON-NLS-1$
				break;
			}
			delete initialValue[attr];
		}

		// keep unknown attributes
		for(var iattr in initialValue) {
			attrSections.normalType.values[iattr] = initialValue[iattr];
		}

		// create sections to edit attributes
		for(var type in attrSections) {
			this._createAttrRows(attrSections[type]);
		}
		
		// set value for Advanced edit field
		this.shapeAdvancedField.value = dojo.toJson(advancedValue);
	},
	
	_isMultilineInput: function(shapeType, attr) {
		return shapeType == "mltext" && attr == "text"; //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	_getShapeType: function(shapeTypeValue) {
		var shapeType = shapeTypeValue;
		if (shapeTypeValue) {
			dojo.forEach(shapesTypes, function(s){
				if(s.type.toLowerCase() == shapeTypeValue.toLowerCase()){
					shapeType = s.type;
					return;
				}
			});
		}
		return shapeType;
	},

	/*
	 * Create sections to edit attributes
	 */
	_createAttrRows: function(section){
		var count = 0;
		for (var attr in section.values) {
			count++;
		}

		var firstRow = true;
		for (var attr in section.values) {
			var tr = dojo.create("tr", null, this.shapeAttrsTable);//$NON-NLS-1$
			
			// Left most td, which shows section name, has rowSpan by number of attributes..
			if (firstRow) {
				if (count > 1) {
					dojo.create("td", {innerHTML: section.label, rowspan:count, style: "white-space: nowrap; vertical-align: top; padding-right: 10px; padding-bottom: 0.5em;", "class": "j-label","id":section.id}, tr);//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$	//$NON-NLS-5$
				} else {
					dojo.create("td", {innerHTML: section.label, rowspan:count, style: "white-space: nowrap; padding-right: 10px; padding-bottom: 0.5em;", "class": "j-label","id":section.id}, tr);//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$	//$NON-NLS-5$
				}
				firstRow = false;
			}
			
			// create tds to show pair of attribute name and value
			this._createAttrRow(section.id, tr, attr, section.values[attr], this._isMultilineInput(section.shapeType, attr));
		}
	},

	_createAttrRow: function(sectionLabelId, trNode, attr, value, isMultiline){
		var self = this;

		var attrLabelId = dijit.getUniqueId("attrLabelId");	//$NON-NLS-1$
		var label = Dictionary.translate("shape", attr);//$NON-NLS-1$
		dojo.create("td", {innerHTML: label || attr, style: "white-space: nowrap; padding-left: 10px; padding-bottom: 0.5em;", "class": "j-label","id":attrLabelId}, trNode);//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$
		var td = dojo.create("td", {style: "width: 100%; padding-left: 10px; padding-bottom: 0.5em;"}, trNode);//$NON-NLS-1$//$NON-NLS-2$
		var input = null;
		if (attrUIExtensions[attr]) {
			if (isMultiline) {
				input = dojo.create("textarea", {style: "width: 86%; resize: vertical;",'aria-labelledby':sectionLabelId+" "+attrLabelId}, td); //$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
			} else {
				input = dojo.create("input", {style: "width: 86%;",'aria-labelledby':sectionLabelId+" "+attrLabelId}, td); //$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
			}
			var titleMsg = attrUIExtensions[attr].title;
			var button = dojo.create("a", { //$NON-NLS-1$
				href: "#", //$NON-NLS-1$
				title: titleMsg,
				className: "dijitInline actionIcon" //$NON-NLS-1$
			}, td);
			this.connect(button, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				attrUIExtensions[attr].editAttribute(input.value, self);
			});
			dojo.create("span", { //$NON-NLS-1$
				className: "dijitInline edit-command" //$NON-NLS-1$
			}, button);
		} else if (isMultiline) {
			input = dojo.create("textarea", {style: "width: 100%; resize: vertical;",'aria-labelledby':sectionLabelId+" "+attrLabelId}, td);//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
		} else {
			input = dojo.create("input", {style: "width: 100%;",'aria-labelledby':sectionLabelId+" "+attrLabelId}, td);//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
		}
		this.connect(input, "onchange", function(e){ //$NON-NLS-1$
			self._updateAdvansedField(self.getValues().value);
		});

		if(value){
			input.value = dojo.isString(value) ? value : dojo.toJson(value);
		}
		this._attrsRow.push({domNode: trNode, attrName: attr, attrValueNode: input});
	},
	
	_createUIExtensions: function() {
		this._addUIExtensions("fill", this.messages.openFillPropertyDialog);//$NON-NLS-1$
		this._addUIExtensions("stroke", this.messages.openStrokePropertyDialog);//$NON-NLS-1$
		this._addUIExtensions("path", this.messages.openPathPropertyDialog);//$NON-NLS-1$
	},

	_addUIExtensions: function(attrName, title) {
		attrUIExtensions[attrName] = {title: title, editAttribute: function(value, self) {
			util.openPicker(attrName, value, title, function(value) {
				self._okPropertyCallback(attrName, value);
			}, self);
		}};
	},
	
	_updateAdvansedField: function(shapeDef) {
		this.shapeAdvancedField.value = dojo.toJson(shapeDef);
	},

	_okPropertyCallback: function(attrName, value) {
		var self = this;
		dojo.some(this._attrsRow, function(a){
			if(a.attrName == attrName) {
				a.attrValueNode.value = value;
				self._updateAdvansedField(self.getValues());
				return true;
			}
		});
	},
	
	getValues: function(){
		var shapeType = this.shapeTypeField.value;
		var values = {
			type: shapeType
		};
		var val = {type: shapeType.toLowerCase()};
		var fontAttrs = null;
		dojo.forEach(this._attrsRow, function(a){
			var attrval = dojo.trim(a.attrValueNode.value);
			//Defect#409858
			attrval=attrval.replace(/[()]/g, '');	//$NON-NLS-1$
			
			if(fontAttributes[a.attrName] != undefined) {
				if (attrval != "") { //$NON-NLS-1$
					if(!fontAttrs) fontAttrs= {};
					fontAttrs[a.attrName] = attrval;
				}
			} else if(attrval){
				try{
					val[a.attrName] = dojo.fromJson(attrval);
				}catch(e){
					val[a.attrName] = attrval;
				}
			}
		});
		if(fontAttrs) {
			val["font"] = fontAttrs; //$NON-NLS-1$
		}
		values.value = val;
		return values;
	}
});

com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog.openDialog = function(shapeDef, okCallback){

	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog({shapeDef:shapeDef});

	var dialogTitle = shapeDef ? contentWidget.messages.configEditorEditShape : contentWidget.messages.AddShape;

	// show dialog
	new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: dialogTitle,
		onOpen: function(){
			dijit.focus(contentWidget.shapeLabelField);
		},
		onOK: function(values){
			var newShapes = values["value"];//$NON-NLS-1$
			if(shapeDef){
				for(var a in shapeDef) delete shapeDef[a]; // clear all
				dojo.mixin(shapeDef, newShapes);
			}
			okCallback({type: values["type"], value: newShapes});//$NON-NLS-1$
		}
	});
};

})();
