/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.Background"); //$NON-NLS-1$

dojo.require("dijit._Templated");//$NON-NLS-1$
dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages = {};

var load_async = jazz.core.loader.load_async;
var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;

var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
	
dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.Background", [dijit._Widget, dijit._Templated, com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin], { //$NON-NLS-1$
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/sections/templates/ConfigFormBackground.html"),  //$NON-NLS-1$  //$NON-NLS-2$

	_shapes: null,
	SECTION_PROP_NAME:"background", //$NON-NLS-1$
	
	postMixInProperties: function(){
		messages = this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$
	},

	postCreate: function() {
		var self = this;
		messages = this.messages;

		this._shapes = [];
		
		var titleNode = dojo.create("span");  //$NON-NLS-1$
		titleNode.appendChild(dojo.doc.createTextNode(this.messages.LabelBackground));
	
		this.inherited(arguments);

		this.shapesToolbar.appendChild(this._createArrowButton(function(){self._moveShape(1, {value: self._shapes}, self._updateShapesTable, self);},
				this.messages.moveShapeDown,
				dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/down.gif"))); //$NON-NLS-1$ //$NON-NLS-2$
		this.shapesToolbar.appendChild(this._createArrowButton(function(){self._moveShape(-1, {value: self._shapes}, self._updateShapesTable, self);},
				this.messages.moveShapeUp,
				dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/up.gif"))); //$NON-NLS-1$ //$NON-NLS-2$
		this.shapesToolbar.appendChild(this._createAddButton(messages.AddShape, self._handleAddShape));
		var updateShape = dojo.hitch(this, this.updateShape);
		this.shapesTable.setupColumns(tableUtils.createTableColumns(
				[
				 {
       		    	id: "type",   //$NON-NLS-1$
       		    	name: messages.configEditorComponentInfoShapeType,
       		    	editable: false, 
       		    	width: "160px"  //$NON-NLS-1$
				 },{
       		    	id: "shape",   //$NON-NLS-1$
       		    	name: messages.configEditorComponentInfoShapeValue,
       		    	editable: false, 
       		    	callback: updateShape,
       		    	preventEmpty: true,
       		    	width: "200px"  //$NON-NLS-1$
				 },{
    		    	id: "actions",  //$NON-NLS-1$
    		    	name: messages.configEditorAction,
    		    	isAction: true
				 }
		]));
		this.setBackground();
	},
	
	setBackground: function(){
		var shapeTable = this.shapesTable;
		shapeTable.clearItems();
		//Defect#409858
		try{
		this._shapes = (this.getConfig().backgroundshape && dojo.fromJson(this.getConfig().backgroundshape)) || [];
		}catch(e){
			this.getConfig().backgroundshape=""; //$NON-NLS-1$
		}
		
		//this._shapes = (this.getConfig().backgroundshape && dojo.fromJson(this.getConfig().backgroundshape)) || [];
		var self = this;
		dojo.forEach(this._shapes, function(shape){
			shapeTable.addItem({
				type : shape.type, 
				shape: dojo.toJson(shape), 
				actions: [{
					iconClass: CLASS_EDIT_COMMAND,
					title: messages.configEditorEditShape,
					onClick: function(e){
						self._editShape(shape);
					}
				},{
					iconClass: CLASS_DELETE_COMMAND,
					title: messages.configEditorRemoveShape,
					onClick: function(e){
						self._deleteItem(this.data.type || this.data.shape, self._shapes, shape);
					}
				}]
			});
		});		
	},

	_findShape: function(shapes, attrJson){
		if(shapes){
			for(var i = shapes.length - 1; i >= 0; i--){
				var shape = shapes[i];
				if(dojo.toJson(shape) === attrJson) return shape;
			}
		}
		return null;
	},
	
	updateShape: function(rowData, propName, newValue, dirtyKey) {
		var shape = this._findShape(this._shapes, rowData.shape);
		if(shape){
			var oldValue = dojo.toJson(shape);
			if(oldValue != newValue){
				for(var i in shape){
					delete shape[i];
				}
				dojo.mixin(shape, dojo.fromJson(newValue)); 
				this.refreshBackground();
			}
		}
	},

	_handleAddShape: function() {
		this._editShape(null);
	},
	
	_editShape: function(shapeDef){
		load_async("com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog", dojo.hitch(this, function() { //$NON-NLS-1$
			com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog.openDialog(shapeDef, dojo.hitch(this, function(newShapeDef){
				if(!shapeDef){
					// newly created
					this._shapes.push(newShapeDef.value);
				}
				this.refreshBackground();
			}));
		}));
	},
	
	_deleteItem: function(itemName, array, item){
		if(itemName){
			if(confirm(dojo.string.substitute(messages.confirmRemoveTargetProperty, [ itemName ]))){
				// remove parameter
				var index = dojo.indexOf(array, item);
				if(index < 0){
					// something wrong
					return;
				}
				array.splice(index, 1);
				this.refreshBackground();
			}
		}
	},
	
	_updateShapesTable: function(shapes) {
		this._shapes = shapes;
		this.refreshBackground();
	},
	
	refreshBackground: function(){
		this.getConfig().backgroundshape = dojo.toJson(this._shapes);
		this._updateDirty(); //$NON-NLS-1$
		this.setBackground();
	},

	setEditMode: function(editMode) {
		if(!editMode) return;

		var mode = editMode.background;
		
		this.backgroundSection.style.display = mode.backgroundSection ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.backgroundContentRow.style.display = mode.backgroundContentRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	startup: function() {
		
	},
	
	destroy: function() {
		this.inherited(arguments);
	}
	
});

})();

