/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.DiagramView"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.layout.TabContainer"); //$NON-NLS-1$
dojo.require("dijit.InlineEditBox"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramEditConfiguration"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramViewInput"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.Layout"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramSidebarContainer"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.ConfigTextEditor"); //$NON-NLS-1$

// sections, do not remove the following statements, otherwise node dialog does not open
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.General"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Query"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Resource"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Connection"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Background"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentInfo"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.SetParametersTable"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.reports.ReportWizard"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.common.web.ui.editor", "PageHelperMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function(){

var util = com.ibm.relm.explorer.web.ui.internal.util;
var messages = dojo.i18n.getLocalization("com.ibm.relm.common.web.ui.editor", "PageHelperMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();

/**
 * DiagramView is a toplevel widget of view hierarchy. 
 * This manages page title, description area, top-level toolbar and diagram area (Layout)
 * 
 * DiagramView 
 *    -(layout)-> Layout 
 *                 -(viewer)-> DiagramWidget 
 *                               -(diagramLayer)-> DiagramEditRoot 
 *                                         -(editor)-> DiagramEditor
 */
var _Layout = "layout";//$NON-NLS-1$ 
var _DiagramWidget = _Layout + ".viewer";//$NON-NLS-1$ 
var _DiagramEditRoot = _DiagramWidget + ".diagramLayer";//$NON-NLS-1$ 
var _DiagramEditor = _DiagramEditRoot + ".editor";//$NON-NLS-1$ 

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.DiagramView", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated, dijit._Container ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/templates/DiagramView.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	title: "", //$NON-NLS-1$
	_descriptionPopup: null,
	page: null,
	permissions: null,
	
	_editorInput: null,
	
	_keptParams: null,
	_keptHash: null,

	postMixInProperties: function() {
		this.inherited(arguments);

		this._keptParams = [];

		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		if(!this.title)
			this.title = this._messages.defaultTitle;
		
		// switch reporting function
		var queryString = dojo.doc.location.search;
		if(queryString.length > 1){
			queryString = queryString.substring(1);
			if(dojo.queryToObject(queryString)["useRPE"] === "true") { //$NON-NLS-1$ //$NON-NLS-2$
				this._useRPE = true;
			}
		}
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		var queryString = dojo.doc.location.search;
		if (queryString.length > 1) {
			queryString = queryString.substring(1);
			if (dojo.queryToObject(queryString)["editSrc"] === "true") { //$NON-NLS-1$ //$NON-NLS-2$
				this.sourcePage = new com.ibm.relm.explorer.web.ui.internal.editor.ConfigTextEditor({title: this._messages.source});
				this.connect(this.sourcePage, "onClose", this.onSourcePageClosed); //$NON-NLS-1$
				this.connect(this.sourcePage, "onSwitch", this.onSwitchToSource); //$NON-NLS-1$
				this.connect(this.sourcePage, "onSaveCompleted", this.onSourcePageSaveCompleted); //$NON-NLS-1$
				this.stackContainer.addChild(this.sourcePage);
			}
		}
		
		// enable/disable edit button
		if (!this._canEdit()) {
			this.editButton.domNode.style.display = "none"; //$NON-NLS-1$
		}
		if (util.checkURLOption("printButton")) {//$NON-NLS-1$
			this.printButton.domNode.style.display = ""; //$NON-NLS-1$
		}
		this.refreshDiagram();

		this.connect(this.stackContainer, "selectChild", this._onSelectChild); //$NON-NLS-1$
		this.connect(this.titleNode, "onChange", this._onTitleChange); //$NON-NLS-1$

		this._editorInput = new com.ibm.relm.explorer.web.ui.internal.view.DiagramViewInput({id: this.viewUri.split("/").pop(), resourceUri: this.viewUri}); //$NON-NLS-1$
		this._editorInput.editor = this;
	},
	_onTitleChange: function(newName){
		var config = dojo.clone(this._getConfig());
		var oldName = config.diagramName;
		if(oldName !== newName){
			var self = this;
			config = dojo.mixin(config, {diagramName: newName});
			var cmd = new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateConfigCommand(this, config, function(){
				self.onConfigUpdated();
				self.page.resizeView();
			});
			this.getDiagramEditor().executeCommand(cmd);
			this._getDiagramEditRoot().onConfigUpdated(); // Update name field in page properties pane
		}
	},
	
	destroy: function(){
		if(this.layout) {
			this.layout.destroy();
			delete this.layout;
		}
		if (this.sourcePage) {
			this.sourcePage.destroy();
			delete this.sourcePage;
		}
		if (this._editorInput) {
			delete this._editorInput;
		}
		this._destroyDescription();

		this.inherited(arguments);
	},
	
	_onSelectChild: function(){
		var selectedTab = this.stackContainer.selectedChildWidget;
		// check if switch from source but source page is dirty state
		if(this.sourcePage && selectedTab != this.sourcePage && this.selectedTab == this.sourcePage && this.sourcePage.isDirty()){
			if (!confirm(this._messages.navigateAway + "\n\n" + this._messages.unsavedChanges + "\n\n" + this._messages.okOrCancel)) { //$NON-NLS-1$ //$NON-NLS-2$
				this.stackContainer.selectChild(this.sourcePage);
				return;
			}
			this.sourcePage.clearContent();
		} 
		// 
		if(selectedTab == this.sourcePage){
			this.onSwitchToSource();
		}else if(selectedTab == this.diagramPage){
			this.onSwitchToDesign();
		}
		// keep the current page
		this.selectedTab = this.stackContainer.selectedChildWidget;
	},

	setEditMode: function(editMode) {
		editMode = editMode && this._canEdit() ? true : false; // always boolean 
		if(editMode === this._editMode) 
			return false;

		this._editMode = editMode;
		
		this.printButton.domNode.style.display = !editMode && util.checkURLOption("printButton") ? "" : "none"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		this.reportButton.domNode.style.display = editMode ? "none" : "";//$NON-NLS-1$//$NON-NLS-2$
		this.editButton.domNode.style.display = editMode || !this._canEdit() ? "none" : "";//$NON-NLS-1$//$NON-NLS-2$
		this.saveButton.style.display = editMode ? "" : "none";//$NON-NLS-1$//$NON-NLS-2$
		this.saveCopyButton.style.display = editMode ? "" : "none";//$NON-NLS-1$//$NON-NLS-2$
		this.saveAndCloseButton.style.display = editMode ? "" : "none";//$NON-NLS-1$//$NON-NLS-2$
		this.cancelButton.style.display = editMode ? "" : "none";//$NON-NLS-1$//$NON-NLS-2$
		this.stackContainer.tablist.domNode.style.display = editMode && this.sourcePage ? "" : "none";//$NON-NLS-1$//$NON-NLS-2$
		this.stackContainer.layout();
		this.resize();

		// refresh editor 
		this.layout.setEditMode(editMode);
		this.diagramPage.showSidebar(editMode);

		this._setTitleInlineEdit(editMode);

		return true;
	},

	_setTitleInlineEdit: function(editMode){
		this.titleNode.set("disabled", !editMode); //$NON-NLS-1$
		this.titleNode.set("title", editMode ? this._messages.editorTitleHelp : ""); //$NON-NLS-1$ //$NON-NLS-2$
		if(editMode){
			dojo.addClass(this.titleNode.domNode, "editIndicator"); //$NON-NLS-1$
		}else{
			dojo.removeClass(this.titleNode.domNode, "editIndicator"); //$NON-NLS-1$
		}
	},

	getEditorInput: function() {
		return this._editorInput;
	},
	
	_canEdit: function(){
		if(this.permissions.modifyPermission) 
			return true;
		var config = this._getConfig();
		if(config && config.ownedBy === com.ibm.team.repository.web.client.session.getAuthenticatedContributorUri())
			return true;
		return false;
	},
	
	_getDiagramEditRoot: function(){
		return dojo.getObject(_DiagramEditRoot, false, this);
	},
	
	_getDiagramEditor: function(){
		return dojo.getObject(_DiagramEditor, false, this);
	},
	
	// for public access
	getDiagramEditor: function(){
		return this._getDiagramEditor();
	},
	
	_getConfig: function(){
		var config = null;
		if(!config){
			config = this._getDiagramConfig();
		}
		return config;
	},
	
	_getDiagramConfig: function(){
		var diagramEditor = this._getDiagramEditor();
		if(diagramEditor){
			return diagramEditor.getConfig ? diagramEditor.getConfig() : null;
		}
		return null;
	},
	
	resize: function(size){
		// temporary fix.
		var bounds = dojo.position(this.domNode);
		if(size){
			bounds.h = size.h;
			this.top.resize(bounds);
		}
		bounds.h -= this.sourcePage ? 90 : 60;
		this.layout.resize(bounds);
	},

	refreshDiagram: function(noReload){
		if(dojo.doc.location.hash != this._keptHash){
			this._keptHash = null;
			this._keptParams = []; // clear
		}
		var currentPage = this.stackContainer.selectedChildWidget;
		if(!this.sourcePage || currentPage != this.sourcePage){
			if(!this.layout){
				var configuration = new com.ibm.relm.explorer.web.ui.internal.view.DiagramEditConfiguration({editable: this._canEdit()});
				this._setFocusOn();
				this.layout = new com.ibm.relm.explorer.web.ui.internal.view.Layout({ viewUri: this.viewUri , configuration: configuration});
				this.layout.startup();
				this.layout.layout();
				var diagramEditor = this._getDiagramEditor();
				if(dojo.isFunction(diagramEditor.setMessageArea)){
					diagramEditor.setMessageArea(this.messageArea);
				}
				var editRoot = this._getDiagramEditRoot();
				this.diagramPage.setDiagramEditRoot(editRoot);
				this.diagramPage.setContent(this.layout, true);
				this.diagramPage.viewer = this.layout;
				
				this.connect(editRoot, "onConfigLoaded", this._onConfigLoaded); //$NON-NLS-1$
				this.connect(editRoot, "onConfigUpdated", this.onConfigUpdated); //$NON-NLS-1$
				this.connect(editRoot, "onViewParameterChanged", this._onViewParameterChanged); //$NON-NLS-1$
				this.connect(editRoot, "onSetParameters", this._onSetParameters); //$NON-NLS-1$
				this.connect(editRoot, "onDirtyStateChanged", this.updateDirty); //$NON-NLS-1$
			} else if(!noReload) {
				// call DiagramEditRoot.loadDiagram
				this._setFocusOn();
				this._getDiagramEditRoot().loadDiagram(this.viewUri);
			}
		}
	},
	
	refreshDiagramLocal: function(focusOn){
		if(focusOn){
			this._setFocusOn();
		}
		var source = this._getConfig();
		this._getDiagramEditRoot().reloadLocal(source, true);
	},

	/**
	 * Set focus to specific node
	 */
	_setFocusOn: function() {
		var hash = dojo.doc.location.hash;
		if(hash && hash.length > 1){
			hash = encodeURI(hash);
			var params = dojo.queryToObject(hash.substring(1));
			if(params.focusOn || params.focusOnText) {
				this._prevSearchText = params.focusOnText;
				if (this._resourceLoadedHandler)
					this.unsubscribe(this._resourceLoadedHandler);
				this._resourceLoadedHandler = this.subscribe("com.ibm.relm.explorer.web.VIEW_LAYOUT_COMPLETED", dojo.hitch(this, function(message) { //$NON-NLS-1$
					var editRoot = this._getDiagramEditRoot();
					if (message && message == editRoot.editor) {
						editRoot.focusOn({focusOn: params.focusOn, focusOnText: params.focusOnText});
					}
				}));
			}
		}
	},
	
	/**
	 * Called when config is loaded in editor. 
	 * The scopes indicate which config section has been loaded. If it is null, all config has been reloaded.
	 */
	_onConfigLoaded: function(config, scopes) {
		if (scopes)
			return;

		this.onConfigUpdated();
		if(this._editMode) {
			dojo.removeClass(this.titleNode.domNode, "jazz-app-ua-TextHoverHelp"); //$NON-NLS-1$
		}else{
			if(config.description && dojo.trim(config.description).length > 0){
				dojo.addClass(this.titleNode.domNode, "jazz-app-ua-TextHoverHelp"); //$NON-NLS-1$
				dojo.removeClass(this.titleNode.domNode, "dijitInlineEditBoxDisplayMode"); //$NON-NLS-1$
				dojo.style(this.titleNode.domNode, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}

		// update edit button
		this.editButton.domNode.style.display = this._editMode || !this._canEdit() ? "none" : "";//$NON-NLS-1$//$NON-NLS-2$
		
		var _config = this._getDiagramConfig();
		this.onConfigLoaded();
		this._editorInput._viewConfigLoaded(_config);
	},
	
	onConfigUpdated: function(){
		// update title and description
		var config = this._getDiagramConfig();
		var diagramName = config.diagramName;
		this.titleNode.set("value", util.unescapeHtml(diagramName)); //$NON-NLS-1$

		this._setDescription(config.description);
		JazzUIWorkbench.setTitle(util.unescapeHtml(diagramName));
		this._editorInput._viewConfigUpdated(config);
	},

	onConfigLoaded: function(){
		// for listeners
		this.page.pageHelper.openEditor(this);
	},
	
	_setDescription: function(description){
		this._destroyDescription();
		if(description && dojo.trim(description).length > 0) {
			this._descriptionPopup = new jazz.ui.HoverPopup({
				around: this.titleNode.domNode,
				html: util.escapeHtml(description),
				openDelay: 300
			});
		}
	},

	_destroyDescription: function() {
		if (this._descriptionPopup) {
			dojo.removeClass(this.titleNode.domNode, 'jazz-app-ua-TextHoverHelp'); //$NON-NLS-1$
			this._descriptionPopup.destroy();
			delete this._descriptionPopup;
			this._descriptionPopup = null;
		}
	},
	
	_onViewParameterChanged: function(viewParams, containerParams) {
		var isParamExist = viewParams && !util.isEmptyObject(viewParams);
		if (!isParamExist)
			isParamExist = containerParams && !util.isEmptyObject(containerParams);

		this.setParamButton.set('enabled', isParamExist);//$NON-NLS-1$
	},
	
	_onSaveCore: function(closeOnSave, saveCopy){
		this._keptParams = []; // clear
		
		this._clearDirty();
		var selectedTab = this.stackContainer.selectedChildWidget;
		if(selectedTab == this.sourcePage){
			this.sourcePage.save();
		}else{
			this._doSave(closeOnSave, saveCopy);
		}
	},
	
	_doSave: function(closeOnSave, saveCopy) {
		// put save request to server
		JazzUIWorkbench.showMessage(this._messages.saving);
		var config = this._getDiagramConfig();

		if(!config.diagramName){
			JazzUIWorkbench.hideMessageArea();
			var message = {
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: this._messages.errorBlankName,
				closable: true
			};
			this.messageArea.showMessage(message);
			return;
		}

		var scope = (config.shared) ? "personal" : "shared"; //$NON-NLS-1$ //$NON-NLS-2$
		var self = this;
		var errorHandler =  function(response, ioargs) {
			self._disableSaveButtons(false);
			JazzUIWorkbench.hideMessageArea();
			var message = util.getXhrErrorMessage(response, ioargs);
			self.messageArea.showMessage(message);
			console.error(message);
		};
		this._disableSaveButtons(true);
		if(saveCopy) {
			var url = com.ibm.relm.explorer.web.ui.configServiceUri;
			var oldViewUri = config.viewUri;
			config.diagramName = dojo.string.substitute(this._messages.copyOf, [ config.diagramName ]);

			jazz.client.xhrPost({
				headers: {
					"Content-Type": "text/json" //$NON-NLS-1$ //$NON-NLS-2$
				},

				url: url,
				postData: this.getConfigForSaveCopy(config),
				load: function(response, ioArgs) {
					self._disableSaveButtons(false);
					JazzUIWorkbench.hideMessageArea();
					self.getDiagramEditor()._eTag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
					self.viewUri = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
					self.page.onSaveCopyView(oldViewUri, self.viewUri);
					self._onSaveCompleted(closeOnSave);
					self.page.setListDirty(scope); // for view list to be refreshed
					self.refreshDiagram();
				},
				error: errorHandler
			});
		} else {
			jazz.client.xhrPut({
				url: config.viewUri,
				headers: {
					"Content-Type": "text/json", //$NON-NLS-1$ //$NON-NLS-2$
					"Etag": this.getDiagramEditor()._eTag //$NON-NLS-1$
				},
				putData: this.getConfigForSave(config),
				load: function(response, ioArgs) {
					self._disableSaveButtons(false);
					JazzUIWorkbench.hideMessageArea();
					self.getDiagramEditor()._eTag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
					self._onSaveCompleted(closeOnSave);
					self.page.setListDirty(scope); // for view list to be refreshed
				},
				error: errorHandler
			});
		}
	},

	_onSaveCompleted: function(/* boolean */ exitEditMode){
		this.updateDirty();
		if(exitEditMode){
			// exit edit mode
			var reloaded = this.setEditMode(false);
			this.stackContainer.selectChild(this.diagramPage);
			// reload diagram view
			this.refreshDiagram(reloaded);// setEditMode reloaded the diagram content
		}
	},

	_disableSaveButtons: function(disable){
		this.saveButton.disabled = disable;
		this.saveCopyButton.disabled = disable;
		this.saveAndCloseButton.disabled = disable;
	},
	
	getConfigForSaveCopy:function(config){
		config.viewUri = null;
		return this.getConfigForSave(config);
	},

	getConfigForSave:function(config){
		// remove system parameter values in saved configuration
		var configClone = dojo.clone(config);
		var params = configClone.params;
		for(var name in params) {

			var param = params[name];
			var type = param.type;
			if(type && type.indexOf("system:") == 0) { //$NON-NLS-1$
				param.value = ""; //$NON-NLS-1$
			}
		}
		return dojo.toJson(configClone);
		
	},
	
	isDirty: function() {
		return this._getDiagramEditRoot().isDirty();
	},
	
	updateDirty: function() {
		com.ibm.relm.common.web.ui.editor.PageHelper.updateDirtyStatus(this.getEditorInput().getPageId());
		this.dirtyStateChanged(this.isDirty());
		
	},
	
	dirtyStateChanged: function(isDirty) {
		// handler is connected
	},
	
	_clearDirty: function() {
		var editRoot = this._getDiagramEditRoot();
		if (editRoot) editRoot.clearDirty();
	},

	////////////////////////////////////////////////////////////////////////////////////////////////////
	// UI handler methods
	////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * set parameter value for view button
	 */
	_onSetParameters: function(ev) {
		var editRoot = this._getDiagramEditRoot();
		var table = new com.ibm.relm.explorer.web.ui.internal.view.common.SetParametersTable();
		editRoot.collectParameters(function(parameters) {
			table.setParameters(parameters);
		});
		
		var self = this;
		new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
			primaryTitle: this._messages.setParameters,
			headerContent: {
				mainMessage: this._messages.paramDialogTitle,
				mainHtmlMessage: this._messages.paramDialogText
			},
			dialogContent: table,
			width: "570px", //$NON-NLS-1$
			isClosable: true,
			validate: function(values, callback) {
				// check if mandatory parameter is set or not
				if(!this._checkParams()){
					alert(self._messages.cancelMessage);
					return;
				}
				// all parameter is set correctly
				callback();
			},
			onOK: dojo.hitch(this, function() {
				var parameters = table.parameters;
				if(!this._editMode){
					this._keptParams = parameters;
					this._keptHash = dojo.doc.location.hash;
				}
				table.setParameters(null); // clear
				editRoot.updateParameters(parameters);
			}),
			_checkParams: function(){
				// check if mandatory parameter is set or not
				return dojo.every(table.parameters, function(p){
					var hasvalue = (!p.mandatory) || (p.type == "boolean") || p.value; //$NON-NLS-1$
					if (!hasvalue && p.containerId) {
						for (var i = 0; i < table.parameters.length; i++) {
							var p2 = table.parameters[i];
							if (!p2.containerId && p2.name == p.name) {
								hasvalue = p2.value;
								break;
							}
						}
					}
					return hasvalue;
				});
			}
		});
	},
	
	_findDialog: null,
	_prevSearchText: null,

	/**
	 * Find Text button
	 */
	_onFind: function(ev){
		if(!this._findDialog){
			// Create Dialog
			var self = this;
			var editor = this._getDiagramEditor();
			var content = dojo.create("div", {style:"margin: 10px;"}); //$NON-NLS-1$ //$NON-NLS-2$
			var textBox = dojo.create("input", {name:"findText", style:"width: 100%"}, content); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			var div = dojo.create("div", {style:"text-align: right; white-space: nowrap; padding: 5px 0;"}, content); //$NON-NLS-1$ //$NON-NLS-2$
			var messageArea = dojo.create("span", {style:"float:left;"}, div); //$NON-NLS-1$ //$NON-NLS-2$
			var findButton = dojo.create("input", {type:"button", value:this._messages.find}, div); //$NON-NLS-1$ //$NON-NLS-2$ 
			var closeButton = dojo.create("input", {type:"button", value:this._messages.close}, div); //$NON-NLS-1$ //$NON-NLS-2$ 
			var dialog = this._findDialog = new jazz.ui.Dialog({
				primaryTitle: this._messages.findTextTitle, 
				contentNode:content, 
				modal:false, 
				closable: false, 
				width: "400px" //$NON-NLS-1$
			});
			dijit.focus(textBox);
			if(this._prevSearchText){
				textBox.value = this._prevSearchText;
			}
			// Cache all resource nodes
			var allResourceNodes = [];
			if(dojo.isFunction(editor.getAllResourceNodes)){
				// getAllResourceNodes is asynclonously activated 
				allResourceNodes = editor.getAllResourceNodes();
			}
			
			// Event handlers for the dialog
			dialog.connect(closeButton, "onclick", function(){ //$NON-NLS-1$
				dialog.close();
			});
			var searchText = null;
			var foundResourceNode = [];
			var index = 0;

			// Nodes get stale if they are reloaded while dialog is open.
			if(!this._findHandler){
				this._findHandler = this.subscribe("com.ibm.relm.explorer.web.VIEW_LAYOUT_COMPLETED", dojo.hitch(this, function(message) { //$NON-NLS-1$
					editor = this._getDiagramEditor();
					if (message && message == editor) {
						allResourceNodes = editor.getAllResourceNodes();
						searchText = null;
					}
				}));
			}

			var findFunction = function(){
				messageArea.innerHTML = ""; //$NON-NLS-1$
				if(!textBox.value) return;
				if(textBox.value != searchText){
					// first search
					self._prevSearchText = searchText = textBox.value;
					foundResourceNode = [];
					index = 0;
					if(searchText){
						foundResourceNode = dojo.filter(allResourceNodes, function(node){
							return dojo.isFunction(node.findText) && node.findText(searchText).length > 0;
						});
					}
				}else{
					// search next
					if(++index >= foundResourceNode.length){
						index = 0;
					}
				}
				if(foundResourceNode.length == 0){
					messageArea.innerHTML = self._messages.notFound;
					return;
				}
				found = foundResourceNode[index];
				self._getDiagramEditRoot().panToNode(found);
				dijit.focus(textBox);
			};
			dialog.connect(textBox, "onkeypress", function(ev){ //$NON-NLS-1$
				if(ev.keyCode == dojo.keys.ENTER){
					findFunction();
				}
			});
			dialog.connect(findButton, "onclick", findFunction); //$NON-NLS-1$
			dialog.connect(dialog, "close", function(){ //$NON-NLS-1$
				dialog.destroy();
				self._findDialog = null;
			});
		}
	},
	
	/**
	 * callback method of report button
	 */
	_onReport: function(ev){
		var self = this;
		// get all named queries used in this view
		var viewUri = this.viewUri;
		var viewId = viewUri.split("/").pop(); //$NON-NLS-1$
		var editor = this._getDiagramEditor();
		var allContainers = editor.getContainerSet();
		var queries = dojo.filter(allContainers.map(function(w){
			var containerNode = w._domNode;
			var containerId = containerNode ? containerNode.getAttribute("id") : null; //$NON-NLS-1$
			if(containerId && dojo.isFunction(w.getQuery)){
				var containerName = containerNode.getAttribute("name"); //$NON-NLS-1$
				var query = w.getQuery();
				if(query && query.sparql){
					return dojo.mixin({
						containerId: containerId,
						containerName: containerName
					}, query);
				}
			}
			return null;
		}), function(q){
			return q;
		});

		// open wizard
		var wizard = new com.ibm.relm.explorer.web.ui.internal.reports.ReportWizard({
			_selectedReport: this._selectedReport,
			_reportParameters: this._reportParameters,
			primaryTitle: this._messages.printView,
			viewId: viewId, 
			queriesInView: queries, 
			customProcessFinish: function(){
				if(!self._useRPE){
					// report UUID
					var reportResultURL = "#action=com.ibm.team.reports.viewReport&reportUUID=" + this._selectedReport.reportUUID; //$NON-NLS-1$
					
					// parameters
					dojo.forEach(this._reportParameters, function(param){
						reportResultURL += "&parameterName=" + encodeURIComponent(param.name) + "&parameterValue=" + encodeURIComponent(param.queryUrl || param.value); //$NON-NLS-1$ //$NON-NLS-2$
					});
					// avoid problem described in WI89673
					reportResultURL += "&parameterName=_dummy_&parameterValue=_dummy_";//$NON-NLS-1$
					
					// open report view
					window.location.href = reportResultURL;
				}else{
					var publishServiceUri = net.jazz.ajax._contextRoot + "/relmui/publish"; //$NON-NLS-1$
					var queryUrls = dojo.map(this._reportParameters, function(p){
						return p.queryUrl;
					});
					var queryString = "?queryEndpoints=" + encodeURIComponent(dojo.toJson(queryUrls)) + "&reportID=" + encodeURIComponent(this._selectedReport.reportId); //$NON-NLS-1$ //$NON-NLS-2$
					window.open(publishServiceUri + queryString);
				}
				self._reportParameters = this._reportParameters; // keep to reuse.
				self._selectedReport = this._selectedReport; // keep to reuse.
			}
		});
		wizard.startup();
	},

	/**
	 * Reload button
	 */
	_onReload: function(ev){
		this._getDiagramEditRoot().clearCache();
		this.refreshDiagram();
	},

	/**
	 * Edit the view button
	 */
	_onEdit: function(){
		this.setEditMode(true);
	},
	
	/**
	 * Save button
	 */
	_onSave: function(ev){
		this._onSaveCore(false, false);
	},

	/**
	 * Save and Close button
	 */
	_onSaveAndClose: function(ev){
		this._onSaveCore(true, false);
	},
	
	/**
	 * Save Copy button
	 */
	_onSaveCopy: function(ev){
		this._onSaveCore(false, true);
	},
	
	/**
	 * Cancel button
	 */
	_onCancel: function(ev){
		var isDirty = this.isDirty();
		if(isDirty && !confirm(dojo.string.substitute(messages.confirmCloseIfDirty, [this._getConfig().diagramName]))) {
			return;
		}
		var reloaded = this.setEditMode(false);
		this.stackContainer.selectChild(this.diagramPage);
		if(isDirty){
			this._clearDirty();
			this.refreshDiagram(reloaded);
		}
	},
	
	/**
	 * Print button
	 */
	_onPrint: function(ev){
		var scale = this._getDiagramEditRoot().getScale();
		window.open(this.viewUri + "?relm.printView=true&relm.ajaxmodule=true&relm.hideToolbar=true&relm.zoomLevel=" + scale*100);//$NON-NLS-1$
	},
	
	///////////////////////////////////////////////////////////////////////////////
	// Config Diagram Page
	///////////////////////////////////////////////////////////////////////////////
	showEditor: function(){
		this.openEditorFromView = true;
		this._onEdit();
	},

	onSwitchToDesign: function(){
	},
	
	///////////////////////////////////////////////////////////////////////////////
	// Config Source Page
	///////////////////////////////////////////////////////////////////////////////
	onSwitchToSource: function(){
		if(this.sourcePage){
			if(this.isDirty()){
				if (!confirm(this._messages.navigateAway + "\n\n" + this._messages.unsavedChanges + "\n\n" + this._messages.okOrCancel)) { //$NON-NLS-1$ //$NON-NLS-2$
					this.stackContainer.selectChild(this.diagramPage);
					return;
				}
			}
			this.sourcePage.setDiagramTitle(this.title);
			if(!this.sourcePage.isDirty()){
				this.sourcePage.load(this.viewUri);
			}
		}
	},
	
	onSourcePageSaveCompleted: function(){
		this._onSaveCompleted(true);
	},
	
	onSourcePageClosed: function(){
		this.stackContainer.selectChild(this.diagramPage);
		if(this.openEditorFromView) {
			this.openEditorFromView = false;
			window.location.href = "#action=com.ibm.relm.explorer.web.showViewList"; //$NON-NLS-1$
			return;
		}
		// reload configurations
		if(this.sourcePage && this.sourcePage.configUpdated){
			this.refreshDiagram();
			this.sourcePage.configUpdated = false;
		}
	}
});

})();