/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2016.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

( function(){

dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.Layout"); //$NON-NLS-1$

dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.HorizontalSlider"); //$NON-NLS-1$
dojo.require("dijit.MenuItem"); //$NON-NLS-1$

dojo.require("jazz.ui.Menu"); //$NON-NLS-1$
dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Separator"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.MenuButton"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramWidget"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramEditConfiguration"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.UserPreferenceUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "LayoutMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ZOOM = "Zoom Button"; //$NON-NLS-1$
var OUTLINE = "Outline Button"; //$NON-NLS-1$
var HIGHLIGHT = "Highlight Button"; //$NON-NLS-1$
var FITTOWINDOW = "Fit To Window Button"; //$NON-NLS-1$
var WHEELSCROLL = "Scroll with Mouse Wheel"; //$NON-NLS-1$
var STOP_LOADING = "Stop Loading Button"; //$NON-NLS-1$

var COPY_OBJ = "copy"; //$NON-NLS-1$
var PASTE_OBJ = "paste"; //$NON-NLS-1$
var REMOVE_OBJ = "remove"; //$NON-NLS-1$

var UNDO = "undo"; //$NON-NLS-1$
var REDO = "redo"; //$NON-NLS-1$

var userPrefUtils = com.ibm.relm.explorer.web.ui.internal.view.common.UserPreferenceUtils;

/**
 * Layout manages a diagram widget and a toolbar that controls zoom and diagram options. Usually it is created by DiagramView 
 * but may be created in a viewlet without DiagramView. 
 * 
 * DiagramView
 *    <-(diagramView)- Layout
 *                 -(viewer)-> DiagramWidget
 *                         -(diagramLayer)-> DiagramEditRoot
 *                                       -(editor)-> DiagramEditor
 */
dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.Layout", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$

	templateString: "<div dojoAttachPoint='containerNode' class='relm-view-layout'><table dojoAttachPoint='toolbarContainerNode' role='presentation' cellspacing=0 cellpadding=0 border=0><tr><td><div dojoAttachPoint='sliderNode'></div></td><td width='100%' align='left'><div dojoAttachPoint='toolsNode'></div></td></tr></table><div dojoAttachPoint='borderNode'></div><div dojoAttachPoint='canvas'/></div>", //$NON-NLS-1$
	
	viewUri: null,

	prefix: null,
	viewer: null, // DiagramWidget
	selection: null,

	toolbar: null,
	highlight_button: null,
	stoploading_button: null,
	wheel_button: null,

	initialZoomLevel: 1.0, 
	stopLoadingState: null,

	tools: [{ name: ZOOM, display: true, enabled: true }, 
			{ name: FITTOWINDOW, display: true, enabled: true },
			{ name: OUTLINE, display: true, enabled: true },
			{ name: HIGHLIGHT, display: true, enabled: true },
			{ name: WHEELSCROLL, display: true, enabled: false },
			{ name: STOP_LOADING, display: true, enabled: false}],
	editTools: [{name: COPY_OBJ, display: true, enabled: false, action: null, onSelection: true},
	            {name: PASTE_OBJ, display: true, enabled: false, action: function(editor, e) {
    				var message = {
    						type: "ActionInvokedNotification", //$NON-NLS-1$
    						src: editor,
    						command: "paste", //$NON-NLS-1$
    						arg: null
					};
					dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
	            }, validate: function(editor) {
	            	if (!editor) return false;
	            	var clipboard = editor.clipboard;
	            	return clipboard && clipboard.getAvailableDataFormats(["layout"]).length > 0; //$NON-NLS-1$
	            }, onSelection: false},
	            {name: REMOVE_OBJ, display: true, enabled: false, action: null, onSelection: true},
	            {name: UNDO, display: true, enabled: false, action: function(editor, e) {
    				var message = {
    						type: "ActionInvokedNotification", //$NON-NLS-1$
    						src: editor,
    						command: "undo", //$NON-NLS-1$
    						arg: null
					};
					dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
	            }, onSelection: false},
	            {name: REDO, display: true, enabled: false, action: function(editor, e) {
    				var message = {
    						type: "ActionInvokedNotification", //$NON-NLS-1$
    						src: editor,
    						command: "redo", //$NON-NLS-1$
    						arg: null
					};
					dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
	            }, onSelection: false}],
	
	prefix: "com.ibm.relm.explorer.web.editor.domain", //$NON-NLS-1$

	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "LayoutMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		if (!this.configuration)
			this.configuration = new com.ibm.relm.explorer.web.ui.internal.view.DiagramEditConfiguration();
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		this.subscribe('btp:SelectionChanged', this._updateToolbarForSelection); //$NON-NLS-1$
		this.subscribe('btp:ClipboardChanged', this._updateToolbarForClipboard); //$NON-NLS-1$
		this._createToolbar();
		this._createDiagram();
		
		this.connect(this.containerNode, "onMove", function() { //$NON-NLS-1$
			console.debug("moved"); //$NON-NLS-1$
		});
	},
	
	destroy: function(){
		if(this.viewer){
			this.viewer.destroy();
			this.viewer = null;
		}
		if(this.toolbar){
			this.toolbar.destroyRecursive();
			this.toolbar = null;
		}
		if(this.slider){
			this.slider.destroy();
			this.slider = null;
		}
		if (this._editModeButtons) {
			this._editModeButtons = null;
		}
		if (this._policyHandles) {
			this._policyHandles = null;
		}
		this.inherited(arguments);
	},
	
	_getDiagramEditRoot: function(){
		return dojo.getObject("viewer.diagramLayer", false, this); //$NON-NLS-1$
	},
	
	_getDiagramEditor: function(){
		return dojo.getObject("viewer.diagramLayer.editor", false, this); //$NON-NLS-1$
	},
	
	_createDiagram: function(){
		this.viewer = new com.ibm.relm.explorer.web.ui.internal.view.DiagramWidget({ configuration: this.configuration, viewUri: this.viewUri, webBaseUri: this.webBaseUri}, this.canvas);

		dojo.marginBox( this.viewer.domNode, "content-cell" ); //$NON-NLS-1$
		this.viewer.startup();

		var diagramLayer = this._getDiagramEditRoot();
		if (!diagramLayer)
			return;
		
		this.connect(diagramLayer, "onConfigLoaded", dojo.hitch(this, function(config){//$NON-NLS-1$
			if (this.printView) {
				if (config && config.diagramName) {
					window.document.title = config.diagramName;
				}
			}
			this._updateToolbarForSelection();
			this._updateToolbarForClipboard();
		}));

		if (diagramLayer.getSelectionController) {
			var selectionController = diagramLayer.getSelectionController();
			if (selectionController)
				this.connect(selectionController, "onSelectionChanged", "handleSelection"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		this._connectPolicies();

		this.connect(diagramLayer, "onViewerCreated", this._connectViewport); //$NON-NLS-1$
		this._connectViewport();
		
		if(this.initialZoomLevel != 1.0)
			diagramLayer.setScale(this.initialZoomLevel, 0, 0);

		this.viewer.outline.setVisible(this.outlineState !== undefined ? this.outlineState : userPrefUtils.getOutlineState());
		this._setButtonsVisibile(diagramLayer && diagramLayer.isEditMode());
		
//		var commandProcessor = diagramLayer.editor.getCommandStack();
		this.connect(diagramLayer, "onDirtyStateChanged", this._updateUndoState); //$NON-NLS-1$
	},

	_connectViewport: function() {
		this._disconnectSlider();

		var diagramLayer = this._getDiagramEditRoot();
		if (this._viewportHandle) this.disconnect(this._viewportHandle);
		this._viewportHandle = this.connect(diagramLayer.viewport, "onViewportChanged", dojo.hitch(this, function() { //$NON-NLS-1$
			
			var value = diagramLayer.viewport.getZoom();
			var val = Math.round(100 * value);
			var label = val + "%"; //$NON-NLS-1$
			this.percent.set('label', label); //$NON-NLS-1$	

			if (this.slider.getValue() == value)
				return;
			
			if (this._viewportChangeHandleTimer) clearTimeout(this._viewportChangeHandleTimer);
			this._viewportChangeHandleTimer = setTimeout(dojo.hitch(this, function() {
				this._viewportChangeHandleTimer = null;
				var value = diagramLayer.viewport.getZoom();

				if (this._sliderHandle) this.disconnect(this._sliderHandle);
				this.slider.set("value", value); //$NON-NLS-1$						
				this._connectSlider();
			}), 300);
		}));
				
		this._connectSlider();
	},
	
	_connectSlider: function() {
		this._sliderHandle = this.connect(this.slider, "onChange", dojo.hitch(this, function(value) { //$NON-NLS-1$
			var diagramLayer = this._getDiagramEditRoot();
			if (value != diagramLayer.viewport.getZoom()) {
				diagramLayer.setScale(value);
			}
		}));
	},
	
	_disconnectSlider: function() {
		if (this._sliderHandle) {
			this.disconnect(this._sliderHandle);
			delete this._sliderHandle;
		}
	},
	
	_updateUndoState: function() {
		
		var self = this;
		dojo.forEach(this._editModeButtons, function(b) { 
			if (b.actionType === UNDO ) {
				b.set('enabled', self._getDiagramEditRoot().canUndo()); //$NON-NLS-1$ 
			} else if (b.actionType === REDO) {
				b.set('enabled', self._getDiagramEditRoot().canRedo()); //$NON-NLS-1$ 
			}
		});
	},
	
	_updateToolbarForClipboard: function(message) {
		var clipboard = this._getDiagramEditor().clipboard;
		dojo.forEach(this._editModeButtons, function(b) { 
			if (b.actionType === PASTE_OBJ && (!message || message.clipboard == clipboard)) {
				b.set('enabled', clipboard.getAvailableDataFormats(["layout"]).length > 0); //$NON-NLS-1$ //$NON-NLS-2$
			} 
		});
	},
	
	_updateToolbarForSelection: function(message) {
		this.showQueryButton.set('enabled', false); //$NON-NLS-1$
		this.startAnalysisButton.set('enabled', false); //$NON-NLS-1$
		this.customActionsButton.set('enabled', false); //$NON-NLS-1$
		dojo.forEach(this._editModeButtons, function(b) { 
			if (b.onSelection) {
				b.set('enabled', false); //$NON-NLS-1$
				b.set('title', b.defaultTitle); //$NON-NLS-1$
			}
		}, this);
		
		var customActions = [];

		if (message) {
			if (!message.selection) message.selection = [];
			var selections = dojo.isArray(message.selection) ? message.selection : [message.selection];
			var node = selections.length > 0 ? selections[0] : null;
			var editor = node ? node._viewer : this._getDiagramEditor();
			if(editor && editor === this._getDiagramEditor()){
				var actions = [];
				var actionTarget = node && (node.primaryMenuTarget || selections.length == 1) ? node : editor;
				var viewActions = dojo.isFunction(actionTarget._getAllActions) ? actionTarget._getAllActions() : null;
				if (viewActions) actions = actions.concat(viewActions);
				var editActions = (editor._editMode && dojo.isFunction(actionTarget._getEditActions)) ? actionTarget._getEditActions() : null;
				dojo.forEach(this.editTools, function(tool) {
					if (tool.onSelection) {
						tool.action = tool.selection = tool.label = null;
						dojo.forEach(editActions, function(a) {
							if (a.actionType === tool.name) {
								tool.selection = selections;
								tool.action = a.action;
								tool.label = a.label;
								tool.enabled = dojo.isFunction(a.validate) ? a.validate() : true;
							}
						});
						if (tool.action) {
							dojo.forEach(this._editModeButtons, function(b) { 
								if (b.onSelection && b.actionType === tool.name) {
									b.set('enabled', tool.enabled); //$NON-NLS-1$
									b.set('title', tool.label); //$NON-NLS-1$
								} 
							});
						}
					}
				}, this);
				
				dojo.forEach(actions, function(action){
					
					if (action.actionType === 'showQuery') { //$NON-NLS-1$
						this.showQueryButton.set('enabled', true); //$NON-NLS-1$
						this.showQueryButton.set('onClick', function(e){ action.action(action.nodeWidget, e);} ); //$NON-NLS-1$
					}
					else if (action.actionType === 'startAnalysis') { //$NON-NLS-1$
						this.startAnalysisButton.set('enabled', true); //$NON-NLS-1$
						this.startAnalysisButton.set('onClick', function(e){ action.action(action.nodeWidget, e);} ); //$NON-NLS-1$
					}
					else if (action.actionType === 'custom') { //$NON-NLS-1$
						customActions.push(action);
					}
				}, this);
			}
		}

		if (customActions.length) {
			this.customActionsButton.set('enabled', true); //$NON-NLS-1$

			function menuProvider() {
				var actionsMenu = new jazz.ui.Menu();

				dojo.forEach(customActions, function(action){
					var label = action._isEscaped ? action.label : jazz.util.html.escape(action.label); 
					action._isEscaped = true;
					var menuItem = new dijit.MenuItem({
	                    label: label, 
						onClick: function(e){
							action.action(action.nodeWidget, e);
						}
					});
					actionsMenu.addChild(menuItem);
				}, this);

				return actionsMenu;
			}

			this.customActionsButton.set('menuProvider', menuProvider); //$NON-NLS-1$
		}
	},

	_startAnalysis: function() {
	},

	_showQuery: function() {
	},

	addEditButton: function(label, tool, icon, show) {
		var editor = this._getDiagramEditor();
		button = this.addButton(label, function(e) {
			if (tool.action) tool.onSelection ? tool.action(tool.selections, e) : tool.action(editor, e);
		}, icon, show);
		button.actionType = tool.name;
		button.onSelection = tool.onSelection;
		button.defaultTitle = button.title;
		
		var enabled = tool.enabled;
		if (!tool.onSelection && tool.validate) {
			enabled = tool.validate(editor);
		}
		button.set('enabled', enabled); //$NON-NLS-1$
		this._editModeButtons.push(button);
	},
	
	addButton: function( label, call, icon, show ){
		var visibleLabel = show ? label : null;

		var button = new jazz.ui.toolbar.Button({
			title: label,
			label: visibleLabel,
			onClick: call,
			imgClass: icon });

		this.addTool( button );

		return button;
	},

	_createToolbar: function() {
		this.toolbar = new jazz.ui.Toolbar({}, this.toolsNode);
		if(this.hideToolbar){
			dojo.style(this.toolbarContainerNode, "display", "none");//$NON-NLS-1$ //$NON-NLS-2$
		} else {
			dojo.style(this.toolbar.domNode, "whiteSpace", "nowrap");//$NON-NLS-1$ //$NON-NLS-2$
		}
		dojo.addClass(this.toolbar.domNode, "rmpsToolbar"); //$NON-NLS-1$

		this.toolbar.domNode.style.height = "25px"; //$NON-NLS-1$
		this.toolbar.domNode.style.background = "none repeat scroll 0 0 white"; //$NON-NLS-1$
		this.toolbar.domNode.style.marginTop = "2px"; //$NON-NLS-1$
		this.toolbar.domNode.style.borderBottom = "none"; //$NON-NLS-1$

		this.borderNode.style.width = "100%"; //$NON-NLS-1$
		this.borderNode.style.borderTop = "1px solid #E8E8E8"; //$NON-NLS-1$
		this.borderNode.style.marginTop = "1px"; //$NON-NLS-1$
		this.borderNode.style.paddingTop = "1px"; //$NON-NLS-1$

		this._editModeButtons = [];

		for(var item = 0; item < this.tools.length; item++ ) {
			if(this.tools[item] && this.tools[item].display) {
				switch( this.tools[item].name ){
				case OUTLINE:
					this.addButton(this._messages.outline, dojo.hitch(this, "toggleOutline"), "toggleOutline"); //$NON-NLS-1$ //$NON-NLS-2$
					break;
				case WHEELSCROLL:
					this.wheel_button = this.addButton(this._messages.wheelzoom, dojo.hitch(this, "toggleMouseWheel"), "wheelScrollOn"); //$NON-NLS-1$ //$NON-NLS-2$
					break;
				case ZOOM:
					this.addZoom();
					break;
				case HIGHLIGHT:
					this.highlight_button = this.addButton(this._messages.highlight, dojo.hitch( this, "toggleHighlight"), "highlightOn");//$NON-NLS-1$ //$NON-NLS-2$
					break;
				case FITTOWINDOW:
					this.addButton( this._messages.fitToWindow, dojo.hitch( this, "fitToWindow" ), "fitIcon"); //$NON-NLS-1$ //$NON-NLS-2$
					break;
				case STOP_LOADING:
					this.stoploading_button = this.addButton(this._messages.stopLoading, dojo.hitch(this, "toggleStopLoading"), "stopLoadingStop"); //$NON-NLS-1$ //$NON-NLS-2$
					break;
				default:
					break;
				}
			}
		}

		this.addTool(new jazz.ui.toolbar.Separator());

		this.showQueryButton = this.addButton(this._messages.showQuery, function() {}, 'query-icon'); //$NON-NLS-1$
		this.showQueryButton.set('enabled', false); //$NON-NLS-1$

		this.startAnalysisButton = this.addButton(this._messages.startAnalysis, function() {}, 'analysis-icon'); //$NON-NLS-1$
		this.startAnalysisButton.set('enabled', false); //$NON-NLS-1$
		
		// edit menus
		if (this.editTools && this.editTools.length > 0) {
			var separator = new jazz.ui.toolbar.Separator();
			this._editModeButtons.push(separator);
			this.addTool(separator);
			
			for(var item = 0; item < this.editTools.length; item++ ) {
				var tool = this.editTools[item];
				if(tool && tool.display) {
					var button = null;
					switch( tool.name ){
					case COPY_OBJ:
						this.addEditButton(this._messages.copyLabel, tool, "copyObject"); //$NON-NLS-1$
						break;
					case PASTE_OBJ:
						this.addEditButton(this._messages.pasteLabel, tool, "pasteObject"); //$NON-NLS-1$
						break;
					case REMOVE_OBJ:
						this.addEditButton(this._messages.deleteLabel, tool, "removeObject"); //$NON-NLS-1$
						break;
					case UNDO:
						this.addEditButton(this._messages.undoLabel, tool, "undo"); //$NON-NLS-1$
						break;
					case REDO:
						this.addEditButton(this._messages.redoLabel, tool, "redo"); //$NON-NLS-1$
						break;
					default:
						break;
					}
					if (button) {
						this._editModeButtons.push(button);
					}
				}
			}
		}

		this.customActionsButton = new jazz.ui.toolbar.MenuButton({
			label: this._messages.actions,
			labelPressed: this._messages.actions, // Work around display bug in MenuButton if there is no labelPressed
			destroyOnClose: true,
			imgClass: 'relm-views-icon-customAction', //$NON-NLS-1$
			enabled: false
		});

		this.addTool(new jazz.ui.toolbar.Separator());
		this.addTool(this.customActionsButton);
	},
	
	setEditMode: function(editMode){
		if(!this._getDiagramEditRoot().setEditMode(editMode))
			return;

		this.resetZoom();
		this._resetPolicies();

		this._setButtonsVisibile(editMode);
	},

	_setButtonsVisibile: function(show) {
		dojo.forEach(this._editModeButtons, function(button) {
			dojo.style(button.domNode, "display", show ? "" : "none"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		});
		dojo.style(this.stoploading_button.domNode, "display", show ? "" : "none"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},
	
	fitToWindow: function() {
		this._getDiagramEditRoot().fitToWindow();
	},

	resetZoom: function() {
		this._getDiagramEditRoot().restore();
	},

	addZoom: function() {
		this.slider = new dijit.form.HorizontalSlider({
			name: "slider", //$NON-NLS-1$
			value: 1.0,
			// min/max must be sync with WheelZoom policy which is created in DiagramLayer
			minimum: 0.05,
			maximum: 2.0,
			intermediateChanges: true,
			style: "width:200px;" //$NON-NLS-1$
		}, this.sliderNode);

		dojo.style(this.slider.domNode, {"display": "inline-block", "vertical-align": "middle"}); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
		this.percent = this.addButton( "100%", dojo.hitch( this, "resetZoom" ), "resetZoom", true ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},
	
	handleSelection: function(){
	},

	addTool: function(item){
		if(!this.toolbar){
			console.warn("Layout.toolbar doesn't exist");//$NON-NLS-1$
			return;
		}
		this.toolbar.addChild(item);
	},

	removeTool: function(item){
		item.destroyRecursive();
	},

	layout: function() { },

	resize: function(s) {
		if(!s) {
			s = dojo.marginBox(this.containerNode.parentNode);
		}

		if(s.w <= 0) s.w = 1; // for IE

		if(this.toolbarContainerNode && this.toolbarContainerNode.style.display != "none"){ //$NON-NLS-1$
			s.h = s.h - 12; // height of Toolbar...
		}

		if(this.viewer) {
			this.viewer.resize(s);
		}
	},
	
	toggleOutline: function() {
		if (this.outlineState !== undefined) {
			this.outlineState = !this.outlineState;
			this.viewer.outline.setVisible(this.outlineState);
		} else {
			var newVisible = !userPrefUtils.getOutlineState();
			this.viewer.outline.setVisible(newVisible);
			userPrefUtils.setOutlineState(newVisible);
		}
	},

	/////////////////////////////////////////////////////////////
	// monitoring policy state changes
	/////////////////////////////////////////////////////////////
	_connectPolicies: function() {
		this._connectPolicy("mouseWheel", this.updateMouseWheelButton); //$NON-NLS-1$
		this._connectPolicy("highlightConnections", this.updateHighlightButton); //$NON-NLS-1$
		this._connectPolicy("stopLoading", this.updateStopLoadingButton); //$NON-NLS-1$
	},
	
	_connectPolicy: function(policyKey, update) {
		if (this._policyHandles && this._policyHandles[policyKey]) {
			this.disconnect(this._policyHandles[policyKey]);
			delete this._policyHandles[policyKey];
		}
		
		var diagramLayer = this._getDiagramEditRoot();
		var policy = diagramLayer.getPolicy(policyKey);
		if (policy) {
			if (!this._policyHandles) this._policyHandles = {};
			this._policyHandles[policyKey] = this.connect(policy, "setState", update); //$NON-NLS-1$
			update.call(this, policy.getState());
		}
	},

	_resetPolicies: function() {
	},
	
	_resetPolicy: function(policyKey, state) {
		var diagramLayer = this._getDiagramEditRoot();
		var policy = diagramLayer.getPolicy(policyKey);
		if (policy && policy.getState() !== state)
			policy.setState(state);
	},

	toggleMouseWheel: function() {
		var policy = this._getDiagramEditRoot().getPolicy("mouseWheel"); //$NON-NLS-1$
		if (policy) {
			if (policy.getState() == "zoom") { //$NON-NLS-1$
				policy.setState("move"); //$NON-NLS-1$
			} else {
				policy.setState("zoom"); //$NON-NLS-1$
			}
		}
	},
	
	updateMouseWheelButton: function(state) {
		if (state == "zoom") { //$NON-NLS-1$
			this.wheel_button.set("imgClass", "wheelScrollOff"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			this.wheel_button.set("imgClass", "wheelScrollOn"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},
	
	toggleHighlight: function() {
		var policy = this._getDiagramEditRoot().getPolicy("highlightConnections"); //$NON-NLS-1$
		if (policy)
			policy.setState(!policy.getState());
	},
	
	updateHighlightButton: function(state) {
		if (state) {
			this.highlight_button.set("imgClass", "highlightOn"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			this.highlight_button.set("imgClass", "highlightOff"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},
	
	toggleStopLoading: function() {
		var policy = this._getDiagramEditRoot().getPolicy("stopLoading"); //$NON-NLS-1$
		if (policy)
			policy.setState(!policy.getState());
	},
	
	updateStopLoadingButton: function(state) {
		if (state) {
			this.stoploading_button.set("imgClass", "stopLoadingPlay"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stoploading_button.set("title", this._messages.startLoading); //$NON-NLS-1$
		} else {
			this.stoploading_button.set("imgClass", "stopLoadingStop"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stoploading_button.set("title", this._messages.stopLoading); //$NON-NLS-1$
		}
	}
});

})();
