/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights: Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.SelectViewDialogContent"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.ViewClient"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.Tree"); //$NON-NLS-1$
dojo.require("jazz.ui.tree.IconItem"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container

var ServiceResponseHandler = com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler;
var ViewClient = com.ibm.relm.explorer.web.ui.internal.view.ViewClient;

/**
 * Widget to select a view
 */
var SelectViewDialogContent = dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.common.SelectViewDialogContent", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/common/templates/SelectViewDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		targetResource: null,
		
		constructor: function(args) {
			dojo.mixin(this, args);

			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
				
		postCreate: function() {
			this.inherited(arguments);
			
			this._responseCount = 0;
			this._saving = false;
			this._allFolders = [];
			this._allViews = [];
			this._selectedView = null;
			// A map mapping view nodes to the underlying view
			this._viewMap = {};
			// A map foder uuid to folder node
			this._folderUuidToFolderNode = {};
			
			this._createFolderTree();
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			this.initTree();

			this._updateEnablement();
		},
		
		initTree: function() {
			this.getFolders("personal"); //$NON-NLS-1$
			this.getFolders("shared"); //$NON-NLS-1$
		},

		// scope : "personal" or "shared"
		getFolders: function(scope) {
			var args = {
					scope: scope,
					projectAreaUUID: this.projectArea || undefined
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders", //$NON-NLS-1$
					"_remoteError"); //$NON-NLS-1$
			ViewClient.getFolderDescriptors(responseHandler, args);
		},
		
		
		getValues: function() 
		{
			return {view: this._selectedView};
		},
		
		_remoteError: function(error) 
		{
			this._movingTypeCount--;

			// Check if the content is still displaying
			if (!dojo.isDescendant(this._okButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this._showErrorMessage(this.messages.invokeServiceError + error.message);
			this._saving = false;
		},

		_getFoldersFromResult: function(response){

			return dojo.map(response.results.bindings, function(r){
				return { 
					"name": { //$NON-NLS-1$
						label: r.name.value, 
						type: "uri", //$NON-NLS-1$
						value: r.uri.value
					}, 
					"description": r.description.value, //$NON-NLS-1$
					"creator": r.creator.value, //$NON-NLS-1$
					"modified": r.modified.value, //$NON-NLS-1$
					"ownedBy": r.ownedBy.value, //$NON-NLS-1$
					"shared" : (r.scope.value == "shared") ? true: false, //$NON-NLS-1$ //$NON-NLS-2$
					"parentUUID": r.parentUuid.value, //$NON-NLS-1$
					"folderUUID": r.folderUUID.value //$NON-NLS-1$
				};
			});
		},

		_addFolders: function(response) {
			var folders = this._getFoldersFromResult(response);
			this._responseCount++;
			if (!folders) folders = [];
			for (var i = 0; i < folders.length; i++) {
				this._allFolders.push(folders[i]);
			}
			if (this._responseCount == 1) {
				return;
			}
			this._addFoldersToTree();
			
			if(!this.targetResource) {
				this.getViews();
			} else {
				this.getRelatedViews();
			}
		},
		
		_addFoldersToTree: function() {
			// First, find the two root folders
			var sharedRoot = null;
			var privateRoot = null;
			for (var i = 0; i < this._allFolders.length; i++) {
				var folder = this._allFolders[i];
				if (folder.parentUUID == "") { //$NON-NLS-1$
					if (folder.shared) {
						sharedRoot = folder;
					} else {
						privateRoot = folder;
					}
				}
			}

			if (privateRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: this.messages.myQueries, iconClass: "iconFolder"}); //$NON-NLS-1$
				privateRoot.treeItemID = rootItem.getId(); //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderUuidToFolderNode[privateRoot.folderUUID] = rootItem;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, privateRoot, this._allFolders, false, false);
			}
			if (sharedRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: this.messages.sharedQueries, iconClass: "iconFolder"}); //$NON-NLS-1$
				sharedRoot.treeItemID = rootItem.getId(); //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderUuidToFolderNode[sharedRoot.folderUUID] = rootItem;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, sharedRoot, this._allFolders, false, true);
			}
		},
		
		/**
		 * From the list of folders, find any that are children of parentFolder, and
		 * add them to the tree. Then recurse for each of the children.
		 */
		_addChildrenForFolder: function(parentFolderNode, parentFolder, folders, parentIsRoot, parentIsShared) {
			var foldersToAdd = [];
			for (var i = 0; i < folders.length; i++) {
				// Special case where the parent is a root system folder
				if (parentIsRoot) {
					if (!folders[i].parentUUID && (folders[i].shared == parentIsShared) && (folders[i].name.label != parentFolder.name.label)) {
						foldersToAdd.push(folders[i]);
					}
				} else {
					if (folders[i].parentUUID == parentFolder.folderUUID) {
						folders[i].shared = parentFolder.shared;
						foldersToAdd.push(folders[i]);
					}
				}
			}
			foldersToAdd = foldersToAdd.sort(function(a, b) {return a.name.label > b.name.label;});
			for (var i = 0; i < foldersToAdd.length; i++) {
				var childNode = new jazz.ui.tree.IconItem({label: com.ibm.relm.explorer.web.ui.internal.util.escapeHtml(foldersToAdd[i].name.label), iconClass: "iconFolder"}); //$NON-NLS-1$
				foldersToAdd[i].treeItemID = childNode.getId(); //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderUuidToFolderNode[foldersToAdd[i].folderUUID] = childNode;
				parentFolderNode.addChild(childNode);
				this._addChildrenForFolder(childNode, foldersToAdd[i], folders, false, parentIsShared);
			}
		},
		
		getViews: function(){
			var self = this;
			self._views = [];
			var projectAreaUUID = self.projectArea || jazz.app.context.get().itemId || ""; //$NON-NLS-1$
			jazz.client.xhrGet({
				url: com.ibm.relm.explorer.web.ui.configServiceUri+"?allconfig", //$NON-NLS-1$ 
				content: {
					projectAreaUUID: projectAreaUUID || "" //$NON-NLS-1$
				},
				load: function(response, ioArgs){
					var templates = response.results.bindings;
					if(templates && templates.length > 0) {
						dojo.forEach(templates, function(t){
							var title = t.title ? t.title.value : t.viewd.value;
							self._views.push({title: title, viewid: t.viewid.value, uri: t.uri.value, parentUUID: t.parentUUID.value});
						});
					}
				},
				handle: function(){
					if(self.defaultEntry){
						self._views.unshift(self.defaultEntry);
					}
					self._addViewToFolder();
				},
				handleAs: "json" //$NON-NLS-1$
			});
		},
		
		getRelatedViews: function(){
			var self = this;
			self._views = [];
			var projectAreaUUID = self.projectArea || jazz.app.context.get().itemId || ""; //$NON-NLS-1$
			jazz.client.xhrGet({
				url: com.ibm.relm.explorer.web.ui.configServiceUri+"?method=relatedConfigDescriptors", //$NON-NLS-1$ 
				content: {
					projectAreaUUID: projectAreaUUID || "", //$NON-NLS-1$
					targetUri: self.targetResource.uri
				},
				load: function(response, ioArgs){
					self._views = response.results.viewProperties;
				},
				handle: function(){
					self._addViewToFolder();
				},
				handleAs: "json" //$NON-NLS-1$
			});
		},

		_addViewToFolder: function() {
			var viewsToAdd = this._views;
			for (var i = 0; i < viewsToAdd.length; i++) {
				var title = viewsToAdd[i].title || viewsToAdd[i].viewid;
				var viewNode = new jazz.ui.tree.IconItem({label: title, iconClass: "iconView"}); //$NON-NLS-1$
				viewsToAdd[i].treeItemID = viewNode.getId(); //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._viewMap[viewNode] = viewsToAdd[i];
				var parentFolderNode = this._folderUuidToFolderNode[viewsToAdd[i].parentUUID];
				if(parentFolderNode){
					parentFolderNode.addChild(viewNode);
				}
			}
			
			if(this.targetResource) {
				// remove empty folders when target resource is specified.
				this._removeEmptyFolder();
			}

		},
		
		_createFolderTree: function() {
			this._folderTreeWidget = new jazz.ui.Tree();
			this._viewTree.appendChild(this._folderTreeWidget.domNode);
			this._folderTreeWidget.onSelectionChanged = dojo.hitch(this,this._handleFolderSelectionChange);
		},
		
		_removeEmptyFolder: function() {
			var self = this;
			dojo.forEach(this._folderTreeWidget.getChildren(), function(item){
				if(item.iconClass == "iconFolder") { //$NON-NLS-1$
					self._checkFolder(item);
				}
			});
		},
		
		_checkFolder: function(folder) {
			var self = this;
			var numChildren = 0;
			var children = folder.getChildren();
			for(var i=children.length-1; i>=0; i--) {
				if(children[i].iconClass == "iconFolder") { //$NON-NLS-1$
					var num = self._checkFolder(children[i]);
					if(num == 0) {
						// remove child folder
						folder.removeChild(children[i], false);
					} else {
						numChildren += num;
					}
				} else if (children[i].iconClass == "iconView") { //$NON-NLS-1$
					numChildren++;
				}
			}
			return numChildren;
		},
		
		_handleFolderSelectionChange: function(selection)
		{
			this._selectedView = this._viewMap[selection];
			this._updateEnablement();			
		},

		updateEnablement: function() {
			this._updateEnablement();
		},
		
		_updateEnablement: function() {
			this.setOkButtonDisabled( this._folderTreeWidget.getSelection().length == 0 ||
										!this._viewMap[this._folderTreeWidget.getSelection()[0]]);
		},
		
		setOkButtonDisabled: function(disable) {
			// dummy for event handler
		},

		_showErrorMessage: function(message) {
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible"); //$NON-NLS-1$
			this.messageBox.showMessage(status);
		},
		
		destroy: function() {
			this.inherited(arguments);
			this._folderTreeWidget.destroy();
		}
	}
);


})();