/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.SetParametersTable"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "CreateNewDiagramWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.common.web.ui.internal.queryparams.paramprompt", "DataPromptMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var util = com.ibm.relm.common.web.ui.internal.util;
var exputil = com.ibm.relm.explorer.web.ui.internal.util;
var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
var enumUtils = com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils;
var HoverPopup = jazz.ui.HoverPopup;

dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.common.SetParametersTable", //$NON-NLS-1$
[dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: "<div class='com-ibm-relm-explorer-setparametertable'><div dojoType='com.ibm.relm.explorer.web.ui.internal.view.common.Table' dojoAttachPoint='_paramTable' inlineEdit='false' style='table-layout: fixed; width:100%;'></div></div>",//$NON-NLS-1$
	
	setParameters: function(params){
		if (!params) {
			this._setParameters(params);
			return;
		}
		enumUtils.loadEnum(dojo.hitch(this, function(enums) {
			this.enums = enums;
			if (dojo.some(params, function(param) {	return enumUtils.isEnumType(param.type, enums); })) {
				this._setParameters(params, enums);
			} else {
				// If there is no enum type in params, call immediately
				this._setParameters(params);
			}
		}));
	},
	
	_setParameters: function(params, enums){
		this.parameters = params;

		var propTable = this._paramTable;
		propTable.clearItems();
		
		var self = this;

		if(this.parameters) {
			dojo.forEach(this.parameters, function(param) {
				var type = param.type;
				var value = param.value;
				
				if (enumUtils.isEnumType(param.type)) {
					value = enumUtils.getEnumValue(param.value);
				}

				var options = null;
				if (param.options){
					options = param.options;
				}else if (enums && enumUtils.isEnumType(type, enums)) {
					var category = enumUtils.getEnumCategory(param.enumtype || type, enums);
					options = enumUtils.getEnumValueOptions(enums, category, self.datamessages.selectValue);
				}

				var readonly = type && type.substr(0, 7) == "system:";//$NON-NLS-1$

				propTable.addItem({name: param.name,
									label: param.label ? param.label : param.name,
									value: value,
									description: param.description,
									readonly: readonly,
									datatypes: {value: type},
									param: param,
									editButton: self._getEditButton(type, value, param.alternativeType),
									buttonTarget: "value"//$NON-NLS-1$
									},
									{value: options},
									function(data, id, newValue, rowWidget) {
										if (id == "value") {//$NON-NLS-1$
											// push newValue back to data
											if (data.param.type == "enum") { //$NON-NLS-1$
												dojo.some(options, function(option) {
													if (option.value == newValue) {
														enumUtils.updateParamByValue(data.param, option.rawvalue);
														return true;
													}
												});
											} else {
												data.param[id] = newValue;
											}
										}
									}, this);
			});
		}
	},
	
	onUpdateParamValue: function(){
	},

	postMixInProperties: function() {
		this.inherited(arguments);
		this._popups = [];
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "CreateNewDiagramWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.datamessages = dojo.i18n.getLocalization('com.ibm.relm.common.web.ui.internal.queryparams.paramprompt', 'DataPromptMessages'); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function(){
		this.inherited(arguments);
		dojo.style(this.domNode, {"table-layout": "fixed"}); //$NON-NLS-1$ //$NON-NLS-2$
		this._initTable();
	},

	destroy: function(){
		this.inherited(arguments);
		dojo.forEach(this._popups, function(popup){popup.destroy();});
	},
	
	_getEditButton: function(type, value, alternativeType){
		return dojo.when(util.hasPicker(type), dojo.hitch(this, function(hasPicker){
			if(hasPicker){
				var button = dojo.create("a", { //$NON-NLS-1$
					href: "#", //$NON-NLS-1$
					title: this.messages.openSelectDialog,
					style: "display: none;", //$NON-NLS-1$
					className: "dijitInline actionIcon" //$NON-NLS-1$
				});
				this.connect(button, "onclick", function(ev){//$NON-NLS-1$
					var initialValue = button.targetColumn ? button.targetColumn.get("value") : value; //$NON-NLS-1$
					util.openPicker(type, initialValue, this.messages.paramSelectParameterValue, function(value){
						if(value.uri){
							value = value.uri; // artifact picker and product picker
						}
						button.targetColumn.set("value", value); //$NON-NLS-1$
						this.close();
					});
					dojo.stopEvent(ev);
				});
				dojo.create("span", { //$NON-NLS-1$
					className: "dijitInline selectButton" //$NON-NLS-1$
				}, button);

				button.editInPlace = true;
				button.style.display = ""; //$NON-NLS-1$
				return button;
			}else if(type != alternativeType && alternativeType){
				return this._getEditButton(alternativeType, value);
			}
		}));
	},

	_initTable: function() {
		var propTable = this._paramTable;

		propTable.setupColumns(this._createColumns());
	},
	
	_createColumns: function() {
		var self = this;
		var columns = tableUtils.createTableColumns([
			{
				id: "label", //$NON-NLS-1$
				name: this.messages.parameterName,
				style: "width: 30%", //$NON-NLS-1$
			},
			{
				id: "value", //$NON-NLS-1$
				name: this.messages.parameterValue,
				style: "width: 70%", //$NON-NLS-1$
				editable: true,
				explictEdit: true
			}
		], this);
		
		columns[0].style = "width: 30%", //$NON-NLS-1$
		columns[1].style = "width: 70%", //$NON-NLS-1$
		
		columns[0].getCellContent = function(cellData) {
			var rowWidget = this;
			var div = dojo.create("span"); //$NON-NLS-1$
			var label = dojo.create("label", {innerHTML : exputil.escapeHtml(cellData)}, div); //$NON-NLS-1$
			var desc = rowWidget.data.description;
			if(desc && desc.length > 0) {
				dojo.addClass(label, 'jazz-app-ua-TextHoverHelp'); //$NON-NLS-1$
				self._popups.push(new HoverPopup({
					around: label,
					html: exputil.escapeHtml(desc),
					openDelay: 300
				}));
			}
			var param = this.data.param;
			if (param.mandatory) {
				var span = dojo.create("span", {innerHTML : '*'}, div); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.addClass(span, "j-required"); //$NON-NLS-1$
			}
			return div;
		};

		columns[0].getCellTitle = function(cellContent, cellData, datatype) {
			var param = this.data.param;
			if (dojo.isString(cellData) && param.containerTitle) {
				return cellData + " (" + param.containerTitle + ")"; //$NON-NLS-2$ //$NON-NLS-1$
			}
			return cellData;
		};

		return columns;
	},

	_updateParamData: function(name, value) {
		if(this.parameters) {
			dojo.forEach(this.parameters, function(param){
				if(param.name == name) {
					param.value = value;
				}
			});
		}
		this.onUpdateParamValue();
	}
});

})();