/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$

(function(){
		
dojo.require("dijit.InlineEditBox"); //$NON-NLS-1$
dojo.require("dijit.form.DateTextBox"); //$NON-NLS-1$
dojo.require("dijit.form.NumberTextBox"); //$NON-NLS-1$
dojo.require("dijit.form.CheckBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Select"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.CascadingComboBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.util"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var escapeHtml = com.ibm.relm.common.web.ui.util.escapeHtml;
var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	
com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils = {};

var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
dojo.mixin(tableUtils, 
{
	/*
	 * Convenience method to create a table column description which contains name, value, description, type, mandatory, actions columns
	 */
	createParamTableColumnsEx: function(nameChangeHandler, valueChangeHandler, disableNameEdit, minimumWidth, scope){
		var columns = [
					{id: "name", name: messages.configEditorParameterName, editable: !disableNameEdit, callback: nameChangeHandler, preventEmpty: true, width: !minimumWidth ? "200px": undefined}, //$NON-NLS-1$ //$NON-NLS-2$
					{id: "value", name: messages.configEditorParameterValue, editable: true, callback: valueChangeHandler, preventEmpty: false, width: !minimumWidth ? "200px": undefined} //$NON-NLS-1$ //$NON-NLS-2$
		];
		if(!minimumWidth){
			columns.push(
					{id: "type", //$NON-NLS-1$
						name: messages.configEditorParameterType, 
						editable: true, 
						options: [
							{label: messages.configEditorParameterType_Normal, value: ""}, //$NON-NLS-1$ 
							{label: messages.configEditorParameterType_Product, value: "product"}, //$NON-NLS-1$ 
							{label: messages.configEditorParameterType_Artifact, value: "artifact"} //$NON-NLS-1$
						],
						callback: valueChangeHandler, 
						preventEmpty: false, 
						width: !minimumWidth ? "150px": undefined}); //$NON-NLS-1$
			columns.push(
					{id: "mandatory", //$NON-NLS-1$
						name: messages.configEditorParameterMandatory, 
						editable: true, 
						options: [
							{label: messages.configEditorParameterMandatory_True, value: true}, 
							{label: messages.configEditorParameterMandatory_False, value: false} 
						],
						callback: valueChangeHandler, 
						preventEmpty: true, 
						width: !minimumWidth ? "100px": undefined}); //$NON-NLS-1$
			columns.push({id: "description", name: messages.configEditorParameterDescription, editable: !disableNameEdit, callback: valueChangeHandler, preventEmpty: false});//$NON-NLS-1$
		}
		columns.push({id: "actions", name: messages.configEditorAction, isAction: true}); //$NON-NLS-1$
		return tableUtils.createTableColumns(columns, scope || this);
	},
	
	/*
	 * Convenience method to create a table column description which contains name, value, description, actions columns
	 */
	createNameValueTableColumns: function(nameChangeHandler, valueChangeHandler, disableNameEdit, valueWidth, noDescription, noAction, scope){
		var columns = [
			{id: "name", name: messages.configEditorParameterName, editable: !disableNameEdit, callback: nameChangeHandler, preventEmpty: true, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
			{id: "value", name: messages.configEditorParameterValue, editable: true, callback: valueChangeHandler, preventEmpty: true, width: valueWidth || "200px"} //$NON-NLS-1$ //$NON-NLS-2$
		];
		if(!noDescription){
			columns.push({id: "description", name: messages.configEditorParameterDescription, editable: !disableNameEdit, callback: valueChangeHandler, preventEmpty: false});//$NON-NLS-1$
		}
		if(!noAction){
			columns.push({id: "actions", name: messages.configEditorAction, isAction: true}); //$NON-NLS-1$
		}
		return tableUtils.createTableColumns(columns, scope || this);
	},
	
	/*
	 * columnDefArray: [ {id, name, editable, callback, preventEmpty, options, width, isAction}, ... ]
	 */
	createTableColumns: function(columnDefArray, scope){
		scope = scope || this;
		return dojo.map(columnDefArray, function(columnDef) { 
			return tableUtils.createTableColumnDescription(columnDef, scope); 
		}, scope);
	},
	
	createTableColumnDescription: function(columnDef, scope){
		var col = {
			id: columnDef.id,
			name: columnDef.name,
			hoverText: columnDef.hoverText,
			isAction: columnDef.isAction
		};
		if(columnDef.editable){
			scope = scope || this;
			var self = tableUtils;
			var callback = dojo.hitch(scope, columnDef.callback);
			col.getCellContent = function(cellData, datatype) {
				var rowWidget = this;
				var cb = this.callback || callback;
				if (datatype == "boolean") { //$NON-NLS-1$
					var checkBox = new dijit.form.CheckBox({name: columnDef.name, checked: cellData, onChange: function(newValue) {
						// call user's call back 
						cb(rowWidget.data, col.id, newValue, rowWidget);
					}});
					return checkBox;
				} else {
					var editorFunc = dojo.hitch(self, col.inlineEdit ? self._createInlineEditBox : self._createValueEditor);
					var editBox = editorFunc(cellData, datatype, function(newValue) {
						// call user's call back 
						cb(rowWidget.data, col.id, newValue, rowWidget);
					}, columnDef.preventEmpty ? rowWidget.data[col.id]: null, columnDef.options, scope);
					return editBox;
				}
			};
			if (columnDef.inlineEdit) {
				col.inlineEdit = columnDef.inlineEdit;
			}
		}
		if(columnDef.width){
			col.style = {width: columnDef.width};
		}
		return col;
	},
	
	_getOption: function(options, value, valueAttr) {
		var option = null;
		if (!valueAttr) valueAttr = "value"; //$NON-NLS-1$
		dojo.some(options, function(opt) {
			if (typeof opt[valueAttr] == "boolean") { //$NON-NLS-1$
				if (opt[valueAttr] == (value == true.toString())) {
					option = opt;
					return true;
				}
			} else if (opt[valueAttr] == value) {
				option = opt;
				return true;
			}
		});
		return option;
	},

	_getInlineEditorClass: function(options, allowUserValue, datatype) {
		if (options) {
			if (allowUserValue)
				return "com.ibm.relm.explorer.web.ui.internal.view.common.CascadingComboBox"; //$NON-NLS-1$
			else
				return "com.ibm.relm.explorer.web.ui.internal.view.common.Select"; //$NON-NLS-1$
		}
		else if (datatype == "date") return "dijit.form.DateTextBox"; //$NON-NLS-2$ //$NON-NLS-1$
		else if (datatype == "number") return "dijit.form.NumberTextBox"; //$NON-NLS-2$ //$NON-NLS-1$
		return "dijit.form.TextBox"; //$NON-NLS-1$
	},

	_createValueEditor: function(cellData, datatype, onChangeCallback, valueForEmpty, options, scope){
		var title = cellData;
		var allowUserValue = false;
		var labelAttr = "label"; //$NON-NLS-1$
		var searchAttr = "value"; //$NON-NLS-1$
		if(options){
			if(dojo.isFunction(options)){
				options = options();
			}else{
				// need copy to set selected 
				options = dojo.clone(options); 
			}
			if (!dojo.isArray(options)) {
				allowUserValue = options.allowUserValue;
				if (options.labelAttr) labelAttr = options.labelAttr;
				if (options.searchAttr) searchAttr = options.searchAttr;
				options = options.values;
			}
			
			var option = this._getOption(options, cellData, searchAttr);
			if (option) {
				option.selected = true;
				title = option[labelAttr];
			}
		}
		
		onChangeCallback = dojo.hitch(scope || this, onChangeCallback);
	
		var editorClass = this._getInlineEditorClass(options, allowUserValue, datatype);
		var ctr = dojo.getObject(editorClass);
		if (!ctr) return;
		
		var params = {};
		
		if (title) {
			params.title = title;
			params.value = title;
		}
		if (options) {
			params.labelAttr = labelAttr;
			params.searchAttr = searchAttr;
			params.optionsArray = options;
		}
		params.datatype = datatype;
		params.onChange = function(value){
			var isEmpty = false;
			if (datatype == "number") { //$NON-NLS-1$
				isEmpty = value !== 0 && !value;
			} else {
				isEmpty = !value || value.length === 0;
			}
			if (isEmpty) {
				if(valueForEmpty){
					alert(messages.configEditorEmptyValue);
					this.set("value", valueForEmpty); //$NON-NLS-1$ 
					return;
				}else{
					value = "";//$NON-NLS-1$
				}
			}
			onChangeCallback(value);
		};

		var editor = ctr(params);
		
		if (options && editor.setOptions)
			editor.setOptions(options);

		editor.connect(editor.domNode, "onkeydown", function(e){ //$NON-NLS-1$
			if(e.keyCode == dojo.keys.ENTER){
				editor.set("value", editor.getValue()); //$NON-NLS-1$ 
			}
		});

		dojo.setStyle(editor.domNode, "width", "100%"); //$NON-NLS-1$ //$NON-NLS-2$
		return editor;
	},
	
	_createInlineEditBox: function(cellData, datatype, onChangeCallback, valueForEmpty, options, scope){
		var title = cellData;
		var allowUserValue = false;
		var labelAttr = "label"; //$NON-NLS-1$
		var searchAttr = "value"; //$NON-NLS-1$
		var titleFunc = null;
		if(options){
			if(dojo.isFunction(options)){
				options = options();
			}else{
				// need copy to set selected 
				options = dojo.clone(options); 
			}
			if (!dojo.isArray(options)) {
				allowUserValue = options.allowUserValue;
				if (options.labelAttr) labelAttr = options.labelAttr;
				if (options.searchAttr) searchAttr = options.searchAttr;
				if (options.getTitle) titleFunc = options.getTitle;
				options = options.values;
			}
			
			var option = this._getOption(options, cellData, searchAttr);
			if (option) {
				option.selected = true;
				title = titleFunc ? titleFunc(option) : option[labelAttr];
			}
		}
		
		onChangeCallback = dojo.hitch(scope || this, onChangeCallback);
		
		var self = this;
		var noValueString = (valueForEmpty) ? "<span class='noValueIndicator'>" + messages.noValueIndicator + "</span>" : "_"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		var editBox = new dijit.InlineEditBox({
			editor: self._getInlineEditorClass(options, allowUserValue, datatype),
			editorParams: {
				optionsArray : options,
				labelAttr: labelAttr,
				searchAttr: searchAttr,
			},
			title : title,
			noValueIndicator: noValueString,
			onChange : function(value){
				if (!value || value.length === 0) {
					if(valueForEmpty){
						alert(messages.configEditorEmptyValue);
						this.set("value", valueForEmpty); //$NON-NLS-1$ 
						return;
					}else{
						value = "";//$NON-NLS-1$
					}
				}

				// replace value into label. 
				// com.ibm....view.common.Select widget shows value instead of label unexpectedly once option is selected.
				// Setting displayedValue in Select widget can resolve this label problem, but if you do so, InlineEditBox cannot distinguish
				// labels which have the same name. As a result, onChange event won't come up if you change option to one which has the same name as current one. 
				if (options) {
					var option = self._getOption(options, value, searchAttr);
					if (option) {
						var label = titleFunc ? titleFunc(option) : option[labelAttr];
						this.domNode.innerHTML = escapeHtml(label);
						if (this.displayNode) this.displayNode.title = label;
					}
				}

//				if (value && datatype == "date") //$NON-NLS-1$
//					value = dojo.date.stamp.toISOString(value);
//
				onChangeCallback(value);
			}
		}, dojo.create("div", { //$NON-NLS-1$
			innerHTML : escapeHtml(title)
		}));
		return editBox;
	}
});

})();