/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2014, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent"); //$NON-NLS-1$

dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.FilteringSelect"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.LinkPathUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "SelectLinkTypeContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container
	var linkPathUtils = com.ibm.relm.explorer.web.ui.internal.view.common.LinkPathUtils;

	/**
	 * Widget to Edit Fill properties
	 */
dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated], 
{ 
	widgetsInTemplate: true,
	templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/SelectLinkTypeContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
	_searching: false,
	_displayOffset: 0,
	_maxDisplayCount: 1000,
	
	constructor: function(){
		this._linkTypeId=dijit.getUniqueId('linkType');//$NON-NLS-1$
	},

	postMixInProperties : function(args) {
		this.inherited(arguments);
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "SelectLinkTypeContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this._searching = true;
	},
		
	postCreate : function() {
		this.inherited(arguments);

		var linkpath = this.props.linkpath[0];
		var sourceTitle = this.props.sourceTitle || linkpath[0].title;
		var targetTitle = this.props.targetTitle || linkpath[linkpath.length-1].title;
		this.selectLinkTypeMessage.innerHTML = jazz.util.html.escape(dojo.string.substitute(this.messages.selectLinkTypeMessage, [sourceTitle, targetTitle]));
		
		if (dojo.some(this.props.linkpath, function(link) { return dojo.isArray(link); })) {
			this.linkTypeLabel.innerHTML = this.messages.linkPathLabel;
		}
		
		this._updateOptions();
		this.setLinkValue(this.props.index, this.props.subindex);

		this.showLineCheck.setChecked(!this.props.hidelines);
		this.linkedNodesOnlyCheck.setChecked(!this.props.unlimited);
		
		this._dropdownHandle = this.connect(this.linkTypeDropdown, "onChange", this.invoke); //$NON-NLS-1$
	},
	
	destroy: function() {
		if (dojo.isFunction(this.closeCallback)) this.closeCallback();
		this._dropdownHandle = null;
		this.inherited(arguments);
	},
	
	invoke: function() {
		var value = this.linkTypeDropdown.get("value"); //$NON-NLS-1$
		if (value == "action.next") { //$NON-NLS-1$
			this._displayOffset += this._maxDisplayCount;
			this._updateOptions();
			this._resetDropdown(false);
			return true;
		} else if (value == "action.prev") { //$NON-NLS-1$
			this._displayOffset -= this._maxDisplayCount;
			if (this._displayOffset < 0) this._displayOffset = 0;
			this._updateOptions();
			this._resetDropdown(false);
			return true;
		} else if (value == "label.searching") { //$NON-NLS-1$
			this._resetDropdown(true);
			return true;
		}
		return false;
	},
	
	_resetDropdown: function(last) {
		if (last) {
			for (var i = this.linkTypeDropdown.store.data.length - 1; i >= 0; i--) {
				var data = this.linkTypeDropdown.store.data[i];
				if (data.value && data.value.length > 6 && data.value.substring(0, 6) == "option") { //$NON-NLS-1$
					this.disconnect(this._dropdownHandle);
					this.linkTypeDropdown.set("value", data.value); //$NON-NLS-1$
					this._dropdownHandle = this.connect(this.linkTypeDropdown, "onChange", this.invoke); //$NON-NLS-1$
					break;
				}
			}
		} else {
			for (var i = 0; i < this.linkTypeDropdown.store.data.length; i++) {
				var data = this.linkTypeDropdown.store.data[i];
				if (data.value && data.value.length > 6 && data.value.substring(0, 6) == "option") { //$NON-NLS-1$
					this.disconnect(this._dropdownHandle);
					this.linkTypeDropdown.set("value", data.value); //$NON-NLS-1$
					this._dropdownHandle = this.connect(this.linkTypeDropdown, "onChange", this.invoke); //$NON-NLS-1$
					break;
				}
			}
		}
	},
	
	setLinkValue: function(index, subindex) {
		this.linkTypeDropdown.set("value", "option." + index + "." + subindex); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},
	
	getLinkValue: function() {
		var value = this.linkTypeDropdown.get("value"); //$NON-NLS-1$
		if (value) {
			var index = value.split("."); //$NON-NLS-1$
			var linkIndex = index[1];
			var subIndex = index[2];
			return this.getLink(this.props.linkpath[linkIndex], subIndex);
		}
		return null;
	},
	
	updateLinkPathList: function(linkpath) {
		this.props.linkpath = linkpath;
		this._updateOptions();
	},
	
	complete: function() {
		this._searching = false;
		if (!this._destroyed)
			this._updateOptions();
	},
	
	_updateOptions: function() {
		var options = this._createOptions();
		var store = new dojo.store.Memory({idProperty: "value", data: options}); //$NON-NLS-1$
		this.linkTypeDropdown.set("labelAttr", "label"); //$NON-NLS-1$ //$NON-NLS-2$
		this.linkTypeDropdown.set("searchAttr", "label"); //$NON-NLS-1$ //$NON-NLS-2$
		this.linkTypeDropdown.set("store", store); //$NON-NLS-1$
	},
	
	_createOptions: function() {
		var options = [];
		if (!this.props.linkpath)
			return options;
		
		var offset = this._displayOffset;
		var maxCount = this._maxDisplayCount;
		var totalCount = 0;
		var addedCount = 0;
		var hasMore = false;
		
		for (var index = 0; index < this.props.linkpath.length; index++) {
			if (maxCount > 0 && addedCount >= maxCount) {
				hasMore = true;
				break;
			}

			var link = this.props.linkpath[index];
			var count = this.getLinkCount(link);
			for (var i = 0; i < count; i++) {
				if (offset <= totalCount++) {
					var expandLink = this.getLink(link, i);
					var label = linkPathUtils.getLinkPathText(expandLink, "title"); //$NON-NLS-1$
					options.push({value: "option." + index + "." + i, label: label, link: expandLink}); //$NON-NLS-1$ //$NON-NLS-2$
					if (maxCount > 0 && ++addedCount >= maxCount)
						break;
				}
			}
		}

		
		options = options.sort(function(a, b) {
			if (a.link.length != b.link.length)
				return a.link.length - b.link.length;
			for (var i = 0; i < a.link.length; i++) {
				var a1 = a.link[i];
				var b1 = b.link[i];
				if (a1.id != b1.id)
					return a1.id.localeCompare(b1.id);
				else if (a1.sourceProperty != b1.sourceProperty)
					return a1.sourceProperty.localeCompare(b1.sourceProperty);
			}
			return 0;
		});

		if (offset > 0 && options.length > 0) {
			var showPrevMessage = dojo.string.substitute(this.messages.prevItemsMessages, [maxCount]);
			options.splice(0, 0, {value: "action.prev", label: showPrevMessage}); //$NON-NLS-1$
		}
		if (hasMore) {
			var showNextMessage = dojo.string.substitute(this.messages.nextItemsMessages, [maxCount]);
			options.push({value: "action.next", label: showNextMessage}); //$NON-NLS-1$
		} else if (this._searching) {
			options.push({value: "label.searching", label: this.messages.searchingMessageEntry}); //$NON-NLS-1$
		}
		return options;
	},

	getLink: function(path, index) {
		if (index == 0)
			return path;
		
		var rest = index;
		var expandPath = dojo.map(path, function(p) {return p;});
		for (var i = path.length - 1; i >= 0 && rest > 0; i--) {
			var p = path[i];
			if (p.branches) {
				var idx = rest % (p.branches.length + 1);
				rest = Math.floor(rest / p.branches.length + 1);
				if (idx > 0) {
					expandPath.splice(i, 1, p.branches[idx - 1]);
				}
			}
			
		}
		return expandPath;
	},
	
	getLinkCount: function(path) {
		var count = 1;
		for (var i = path.length - 1; i >= 0; i--) {
			var p = path[i];
			if (p.branches) {
				count *= (p.branches.length + 1); 
			}
		}
		return count;
	}
});


com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent.openDialog = function(props, okCallback, closeCallback, left, top){

	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent({props: props, closeCallback: closeCallback});

	// show dialog
	var dialog = new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		primaryTitle: contentWidget.messages.selectLinkTypeTitle,
		dialogContent: contentWidget,
		isClosable: true,
		validate: function(values, callback) {
			callback();					
		},
		onOK: dojo.hitch(this, function(){
			var linkpath = contentWidget.getLinkValue();
			var hidelines = !contentWidget.showLineCheck.get("checked"); //$NON-NLS-1$
			var unlimited = !contentWidget.linkedNodesOnlyCheck.get("checked"); //$NON-NLS-1$
			okCallback({linkpath: linkpath, hidelines: hidelines, unlimited: unlimited });
		})
	});
	
	if (left) dialog.domNode.style.left = left + "px"; //$NON-NLS-1$
	if (top) dialog.domNode.style.top = top + "px"; //$NON-NLS-1$
	
	return contentWidget;
};

})();
