/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.xmlGenerator"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

(function(){
	
var util = {
	escapeHtml: jazz.util.html.escape
};

var getIndentSpace = function(indent){
	var indentStr = "";//$NON-NLS-1$
	for(var i=0; i<indent; i++){
		indentStr += "   ";//$NON-NLS-1$
	}
	return indentStr;
};

var escapeQuote = function(text){
	return text.split("\"").join("&quot;");//$NON-NLS-1$//$NON-NLS-2$
};

var createContainerParams = function(params, additions, indent){
	var str = "";//$NON-NLS-1$
	// param tags under <container> tag
	for(var name in params) {
		var beginStr = "<param name=\"" + escapeQuote(util.escapeHtml(name)); //$NON-NLS-1$
		var endStr = "\"/>\n"; //$NON-NLS-1$

		var param = params[name];
		var type = param.type;
		var paramStr = ""; //$NON-NLS-1$
		var spacing = "\" "; //$NON-NLS-1$
		for (var key in param) {
			var value = param[key];
			if (value != undefined) {
				if (key == "value" && !dojo.isString(value)) { //$NON-NLS-1$
					if (type == "date") //$NON-NLS-1$
						value = dojo.date.stamp.toISOString(value);
					else if (type == "enum") //$NON-NLS-1$
						value = dojo.toJson(value);
				}
				value = escapeQuote(util.escapeHtml(value));
				paramStr += "\" " + key + "=\"" + value; //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		if (additions) {
			for (var key in additions) {
				var value = additions[key];
				if (value != undefined) paramStr += spacing + key + "=\"" + value; //$NON-NLS-1$ //$NON-NLS-2$
			}
		}

		str += getIndentSpace(indent + 1) + beginStr + paramStr + endStr;
	}
	return str;
};

var generateWidget = function(widget, indent){
	var ml = ""; //$NON-NLS-1$
	
	if(widget.preventSave) return ml;
	
	var widgetName = null;
	var parameters = [];
	var upstream = null;
	if(dojo.isFunction(widget.getOutputTagName)){
	
		widgetName = widget.getOutputTagName();
		if(widgetName){
			ml = getIndentSpace(indent) + '<' + widgetName; //$NON-NLS-1$
			var componentInfo = widget.hlp_getComponentInfo();
			
			var attrs = widget.hlp_getAttributes() || [];
			for(i = 0, len = attrs.length; i < len; i++){
				var attr = attrs[i];
				if(attr.readonly === true){ continue; }
				var name = attr.name;
				var value = widget.hlp_getPropertyValue(name, true);
				if(value && value.length > 0 && !dojo.isArray(value)) {
					value = escapeQuote(util.escapeHtml(value));
				}
				
				var writeValue = false;
				if(widgetName == "container" && name == "parameters") { //$NON-NLS-1$ //$NON-NLS-2$
					if(value) parameters.push({params: value, additions: {use: widget.paramUses.defaultUse}});
				} else if(widgetName == "container" && name == "uiParams") { //$NON-NLS-1$ //$NON-NLS-2$
					if(widget.paramUses.ui && value) parameters.push({params: value, additions: {use: widget.paramUses.ui}});
				} else if (widgetName == "container" && name == "upstream") {//$NON-NLS-1$ //$NON-NLS-2$
					if(value) upstream = value;
				} else if (componentInfo && typeof value == "boolean") {//$NON-NLS-1$
					// In case specified value is boolean type and is not default value, write it.
					dojo.some(componentInfo.attributes, function(attribute) {
						if (attribute.name == name) {
							writeValue = attribute.value === undefined ? value : attribute.value != value;
							return true;
						}
					});
				} else if(value){
					writeValue = true;
				}
				
				if (writeValue) {
					ml +=  " " + name + "=\"" + value + "\""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}
			}
			ml += ">\n";//$NON-NLS-1$
		}
	}
	
	dojo.forEach(parameters, function(p) {
		ml += createContainerParams(p.params, p.additions, indent);
	});

	// upstream tags under <container> tag
	dojo.forEach(upstream, function(link) {
		ml += getIndentSpace(indent + 1) + "<upstream id='" + util.escapeHtml(link.id) + "' linktype='" + link.linktype; //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-1$
		if (link.lineonly) ml += "' unlimited='" + link.unlimited; //$NON-NLS-1$
		if (link.hidelines) ml += "' hidelines='" + link.hidelines; //$NON-NLS-1$
		if (link.connectionId) ml += "' connectionId='" + util.escapeHtml(link.connectionId); //$NON-NLS-1$
		ml += "'/>\n"; //$NON-NLS-1$
	});
	
	var children = widget.getChildren ? widget.getChildren() : null;
	if(children){
		for(var i = 0, len = children.length; i < len; i++){
			ml += generateWidget(children[i], indent + 1);
		}
	}
	
	if(widgetName){
		ml += getIndentSpace(indent) + '</' + widgetName + '>\n'; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	return ml;
};


com.ibm.relm.explorer.web.ui.internal.view.extensions.xmlGenerator = {
	generate: function(widget){
		widget = widget || this;
		var ml = generateWidget(widget, 0);
		return "<layout xmlns='http://relm.ibm.com/view/0.9/'>\n" + ml + "</layout>";  //$NON-NLS-1$ //$NON-NLS-2$ 
	},
	
	_findRootContainer: function(widget){
		if(widget.componentInfoId === "container") return widget;//$NON-NLS-1$
		var children = widget.getChildren ? widget.getChildren() : null;
		var container = null;
		if(children){
			for(var i = 0, len = children.length; i < len && !container; i++){
				container = this._findRootContainer(children[i]);
			}			
		}
		return container;
	}	
};

})();