/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes.contextMenuHelper"); //$NON-NLS-1$

dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$

(function(){
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

com.ibm.relm.explorer.web.ui.internal.view.nodes.contextMenuHelper = {

		/**
		 * Creates and shows a self-destroying context menu with all the Actions
		 * available on this node. If no coords are given, the menu will open below this
		 * node.
		 * 
		 * @param  {int?} x x coordinate to show menu at, likely event.clientX. Optional.
		 * @param  {int?} y y coordinate to show menu at, likely event.clientY. Optional.
		 * @return {jazz.ui.Menu}
		 */
		showContextMenu: function(x, y) {
			var actions = [];
			var editor = this._viewer;
			if (!editor && this.hlp_getEditor)
				editor = this.hlp_getEditor();
			
			if (editor) {
				var selections = editor.focusWidgets ? editor.focusWidgets : [];
				var actionTarget = this.primaryMenuTarget || selections.length < 2 ? this : editor;
				var viewActions = dojo.isFunction(actionTarget._getAllActions) ? actionTarget._getAllActions() : null;
				if (viewActions) actions = actions.concat(viewActions);
				if (editor._editMode) {
					var editActions = dojo.isFunction(actionTarget._getEditActions) ? actionTarget._getEditActions() : null;
					if (editActions) actions = actions.concat(editActions);
				}
				var viewerActions = editor.getContextMenuItems ? editor.getContextMenuItems(this) : [];
				if(actions.length > 0 && viewerActions.length > 0){
					actions.push("separator"); //$NON-NLS-1$
				}
				actions = actions.concat(viewerActions);
				if (!actions.length > 0) return false;

				if (!x || !y) {
					// XXX: assumes there is a _viewer property to find the top shape layer.
					var pos = utils.shapePosition(this.shape, editor);
					x = pos.x;
					y = pos.y + pos.h;
				}
			}

			this._richHover && this._richHover.close();
			var menuInfos = this._createMenuInfos(actions);

			return utils.createAndPopupMenu(this, menuInfos, x, y);
		},
		
		_createMenuInfos: function(actions) {
			var defaultIcon = dojo.some(actions, function(item) {return item.iconClass;}) ? "blank-icon" : null; //$NON-NLS-1$
			
			return dojo.map(actions, function(item){
				if (item == 'separator') { //$NON-NLS-1$
					return 'separator'; //$NON-NLS-1$
				}
				else if(item.isGroup){
					return {
						label: jazz.util.html.escape(item.label),
						iconClass: item.iconClass || defaultIcon,
						isGroup: true,
						menuInfos: this._createMenuInfos(item.actions)
					};
				}
				else if(item.action){
					var disabled = false;
					if(item.actionString){
						// validate the action is ready to execute. 
						var funcName = item.actionString.substring(0, item.actionString.indexOf('('));//$NON-NLS-1$
						if(dojo.isFunction(this["validate_" + funcName])){//$NON-NLS-1$
							var f = "this.validate_" + item.actionString; //$NON-NLS-1$
							disabled = !eval(f);
						}
					} else if (dojo.isFunction(item.validate)) {
						disabled = !item.validate();
					}
					return {
						label: jazz.util.html.escape(item.label),
						iconClass: item.iconClass || defaultIcon,
						onClick: function(nodeWidget, evt){
							item.action(nodeWidget, evt);
						},
						selected: item.selected,
						disabled: disabled
					};
				}
			}, this);
		}
};

})();