/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.le.leeditor.page.QueryResult"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.Menu"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("dojo.date"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.internal.queryparams.paramprompt.ParameterPromptDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.internal.queryparams.SystemParameterUtil"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$
dojo.require('com.ibm.relm.le.ui.internal.util'); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.view.ResourceList"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.le.leeditor.internal", "QueryResultMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();

var ICONCLASS_ARTIFACT = "artifact-icon"; //$NON-NLS-1$
var ICONCLASS_PROPERTIES = "properties-icon"; //$NON-NLS-1$
dojo.declare("com.ibm.relm.le.leeditor.page.QueryResult", //$NON-NLS-1$
	com.ibm.relm.le.ui.internal.view.ResourceList,
{
	_queryInfo: null,
	previewMode: false,

	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.le.leeditor.internal", "QueryResultMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	postCreate: function(){
		this.inherited(arguments);
		
		this.subscribe("RESOURCE_UPDATED", "onResourceUpdated"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	/**
	* Prompt the user for parameter values. If parameter values are known,
	* then those values will be prefilled
	* 
	* @param existingParameterValues parameterValues that are already known -
	*     to prefil prompt
	*/
	promptForParameters: function(existingParameterValues) {
		console.trace();
		if (this._queryInfo.parameterized && !this._queryInfo.parameters && !this.previewMode) {
			//load parameters
//			var self = this;
//			var responseHandler = new ServiceResponseHandler(this, function(queryInfo) {
//				self._getQueryInfoCallback(queryInfo);
//			}, '_remoteError'); //$NON-NLS-1$
//			ExplorerClient.getQueryInfo(responseHandler, {queryURI: this._queryInfo.queryUri});
		} else {
			this._performPrompt(existingParameterValues);
		}
	},
	
	/**
	 * Navigate to the query editor and pre-populate with the given parameters.
	 */
	_editSubstitutedQuery: function(sparql) {
		var name = dojo.replace(this._messages.newStaticQueryName, [this._queryInfo.queryName]);
		dojo.publish("OPEN_QUERY_EDITOR", [{ //$NON-NLS-1$
			mode: "create", //$NON-NLS-1$
			queryName: name,
			sparql: sparql,
			description: this._queryInfo.description,
			scope: this._queryInfo.scope
		}]);
	},
	
	/**
	 * [_performPrompt description]
	 * @param  {[type]}  existingParameterValues [description]
	 * @return {[type]}                          [description]
	 */
	_performPrompt: function(existingParameterValues) {
		var onSubmit = function(parameterValues) {
			this._queryInfo.parameterValues = parameterValues;
			this._refreshContent(null, true);
		};

		var self = this;
		var isDirty = this._queryInfo.isDirty;

		var onCancel = function() {
			var hashObj = dojo.queryToObject(window.location.hash.substring(1));
			if (hashObj.action == "com.ibm.relm.explorer.web.showExplorer" && (hashObj.edit == "true" || hashObj.mode == "create")) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				self.onEditQuery();
			} else if (isDirty) {
				self.onEditQuery();
			} else {
				if (window.history.length > 1) {
					self.unregisterEvents();
					window.history.back();
				} else {
					self.unregisterEvents();
					window.location.href = "#action=com.ibm.relm.explorer.web.showQueryList"; //$NON-NLS-1$
				}
			}
		};

		var promptArgs = {
			width: 600,
			needAllParameterValues: false,
			parameterValues: existingParameterValues,
			queryName: this._queryInfo.queryName,
			queryUri: this._queryInfo.queryUri,
			isDirty: isDirty,
			_onCancel: dojo.hitch(this, onCancel),
			_onSubmit: dojo.hitch(this, onSubmit),
			_onSaveAs: dojo.hitch(this, this._editSubstitutedQuery)
		};

		if (!dojo.isArray(this._queryInfo.parameters)) {
			var parameters = [];
			for (var parameter in this._queryInfo.parameters) {
				parameters.push(this._queryInfo.parameters[parameter]);
			}
			promptArgs.parameters = parameters;
		} else {
			promptArgs.parameters = this._queryInfo.parameters;
		}

		var d = new com.ibm.relm.common.web.ui.internal.queryparams.paramprompt.ParameterPromptDialog(promptArgs);
		d.startup();
	},

	_getQueryInfoCallback: function(queryInfo) {
		dojo.mixin(this._queryInfo, queryInfo);
		this._performPrompt(null);
	},

	_remoteError: function(error) {
		PlatformUI.getWorkbench().showMessage(error.message);
	},

	setQueryInfo: function(queryInfo){
		this.filterBox.setValue(""); // clear filter box //$NON-NLS-1$
		this._queryInfo = queryInfo;
		this._dirtyMessages = queryInfo._dirtyMessages;
		this._monitorDirtyQuery(this._queryInfo.isDirty, this._dirtyMessages);
		// retrieve the correct pagesize from dropdown
		this.pageSize = this._getPageSize();
		this.pageIndex = 0;
		this._resultuuid = null;
		this.repromptParameters = false;
		this._refreshContent();
	},

	unregisterEvents: function(args) {
		if (this.navigateAwayHandle) {
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
			delete this.navigateAwayHandle;
		}
		if (this.onBeforeUnloadHandle) {
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
			delete this._handleOnBeforeUnload;
		}
	},

	/**
	 * The function is defined in this way to capture the closure scope
	 * 
	 * @param  {boolean} isDirty indicates the query whose results are showing is dirty
	 * @param {Object} dirtyMessages a map of messages to show on the navigate away handler
	 */
	_monitorDirtyQuery: function(isDirty, dirtyMessages) {

		//False means allow navigation away
		this._handleNavigateAway = function(url, e) {
			// Allow help links to open in a new window without ado
			if (e.target && (e.target.tagName == "A") && (e.target.target == "jazzhelp")) { //$NON-NLS-1$ //$NON-NLS-2$
				return false; // allow navigation
			}
			
			var currentLocation = window.location.href;
			var badLink = currentLocation.substring(0, currentLocation.indexOf("#")+1); //$NON-NLS-1$
			if (url == badLink) {
				return true;
			}
			if (!window.confirm(dirtyMessages.navigateAwayMessage + "\n\n" + dirtyMessages.unsavedChangesMessage + "\n\n" + dirtyMessages.okOrCancelMessage)) { //$NON-NLS-1$ //$NON-NLS-2$
				return true; // don't navigate away
			}
			this.unregisterEvents();
			return false; // allow navigation
		};
		
		this._handleOnBeforeUnload = function() {
			return dirtyMessages.unsavedChangesMessage + "\n\n"; //$NON-NLS-1$
		};
		
		if (isDirty) {
			if (!this.navigateAwayHandle) {
				this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(this._handleNavigateAway, /*priority*/ 10000);
			}
			if (!this.onBeforeUnloadHandle) {
				this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(this._handleOnBeforeUnload, /*priority*/ 10000);
			}
		} else {
			if (this.navigateAwayHandle) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				delete this.navigateAwayHandle;
			}
			if (this.onBeforeUnloadHandle) {
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				delete this.onBeforeUnloadHandle;
			}
		}
	},

	/**
	 * Override here to ensure that user is re-prompted for parameters after a
	 * refresh (reload) if this is a parameterized query.
	 */
	onReload: function() {
		this.repromptParameters = true;
		this.inherited(arguments);
	},


	_refreshContent: function(pageUrl, reload) {
		var util = com.ibm.relm.le.ui.internal.util;
		this.inherited(arguments);
		this.messageArea.hideMessage();
		this.showEditButton(false);
		var self = this;
		self._removeContent();
		self._resources = [];
		
		var xhrInfo = this._setupQueryExecution(pageUrl, reload, false);
		if(xhrInfo){
			var url = xhrInfo.url;
			var qinfo = self._queryInfo;
			var method = xhrInfo.method;
			var headers = xhrInfo.headers;
			var hasBody = xhrInfo.hasBody;
			var messages = self._messages;
			var title = ""; //$NON-NLS-1$
			var content = xhrInfo.content;
			
			if(qinfo.resourceUri){
				if(util.isValidUrl(qinfo.resourceUri)){
					self.titleText.innerHTML = "<a href='" + qinfo.resourceUri + "' target='_new'>" + qinfo.resourceUri + "</a>"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}else{
					self.titleText.innerHTML = util.escapeHtml(qinfo.resourceUri);
				}
				var segs = qinfo.resourceUri.split('/'); //$NON-NLS-1$
				if(segs.length > 0)	title = segs[segs.length-1];
			}else{
				if(qinfo.query){
					self.titleText.innerHTML = messages.SearchResult + " : " + util.escapeHtml(qinfo.query); //$NON-NLS-1$
					title = qinfo.query;
				}else{
					self.titleText.innerHTML = messages.QueryResult + " : " + util.escapeHtml(qinfo.queryName || ""); //$NON-NLS-1$ //$NON-NLS-2$
					title = qinfo.queryName;
				}
			}
			JazzUIWorkbench.setTitle(title || ""); //$NON-NLS-1$
			JazzUIWorkbench.showMessage(messages.ExecutingQuery);
			
			jazz.client.xhr(method, {
				url: url,
				headers: headers,
				content: content,
				handleAs: "json", //$NON-NLS-1$
				handle: function(){
					JazzUIWorkbench.hideMessageArea();
				},
				load: function(data, ioArgs){
					self.lastSparql = qinfo.sparql || ""; //$NON-NLS-1$
					var columns = null;
					if(data){
						var noPaging = false;
						if(data.head){
							// result of SELECT
							if (self._queryInfo && self._queryInfo.resourceUri){
								columns = [];
								columns[0] = {id: "property", label: messages.Property, sort: "_by_property", isPredicate: true}; //$NON-NLS-1$ //$NON-NLS-2$
								columns[1] = {id: "value", label: messages.Value, sort: "_by_value"}; //$NON-NLS-1$ //$NON-NLS-2$
							}else{
								var columnsInfo = data.head.vars;
								if(columnsInfo && columnsInfo.length > 0){
									columns = dojo.map(columnsInfo, function(c){
										var ret = {id: c, label: c, sort: "_by_" + c}; //$NON-NLS-1$
										return ret;
									});
								}
							}
							if(data.results){
								self._resources = data.results.bindings;
							}else if(data["boolean"] !== undefined){ //$NON-NLS-1$
								// result of ASK
								self._resources = [{ask: {value: new String(data["boolean"])}}]; //$NON-NLS-1$
								columns = [{id: "ask", label: messages.Boolean, sort: "_by_ask"}]; //$NON-NLS-1$ //$NON-NLS-2$
							}
						}else{
							// result of CONSTRUCT or DESCRIBE
							columns = [];
							columns[0] = {id: "subject", label: messages.Subject, sort: "_by_subject"}; //$NON-NLS-1$ //$NON-NLS-2$
							columns[1] = {id: "predicate", label: messages.Predicate, sort: "_by_predicate", isPredicate: true}; //$NON-NLS-1$ //$NON-NLS-2$
							columns[2] = {id: "object", label: messages.Object, sort: "_by_object"}; //$NON-NLS-1$ //$NON-NLS-2$
							self._resources = [];
							for(var subject in data){
								var predicates = data[subject];
								if(dojo.isString(predicates)) continue;
								for(var predicate in predicates){
									var objects = predicates[predicate];
									for(var i=0; i<objects.length; i++){
										self._resources.push({subject: subject, predicate: predicate, object: objects[i]});
									}
								}
							}
							noPaging = true;
						}
						self._totalResults = data.totalSize || self._resources.length;
						self._updatePaginationBar(self._resources, data.totalSize, noPaging);
						self._resultuuid = data.resultuuid;
						self._sparql = data.sparql;

					}
					// Render resource list
					self._displayResources(true, columns);
				},
				error: function(response, ioargs){
					self.errorHandler.apply(self, arguments); 
					jazz.client.xhrGet({ //Request the sparql after substitution if there is an error
						url: com.ibm.relm.le.ui.queryServiceUri,
						handleAs: "text", //$NON-NLS-1$
						failOk: true,
						content: {
							method: "getSubstitutedQuery", //$NON-NLS-1$
							queryURI: qinfo.queryUri,
							parameterValues: content.parameterValues,
							sparql: qinfo.sparql
						},

						load: function(sparql) {
							self._sparql = sparql;
						},

						error: function(error, ioArgs) {
							self._sparql = qinfo.sparql;
							console.error(dojo.fromJson(ioArgs.xhr.responseText));
						}
					});
				}
			}, hasBody);
		}
	},
	
	/**
	 * Not sure... does a lot of stuff to prepare for a query.
	 * 
	 * Note that if this is a parameterized query and the params are not
	 * already available, this will prompt and actually return `null`. The
	 * prompt dialog will itself rerun the query later, so this particular
	 * gets aborted. Because of that, this methof takes some hacky booleans to
	 * decide what to do after the prompt. Ideally, this would just return a
	 * dojo.Deferred() so the caller can decide what to do, but that's a big
	 * change (this was written during a Release Candidate).
	 * 
	 * @param  {string}   pageUrl       [description]
	 * @param  {boolean}  reload        [description]
	 * @return {object}                some details on how to run the query.
	 *     `null` if it prompts the user for params.
	 */
	_setupQueryExecution: function(pageUrl, reload){
		if(!this._queryInfo){
			return null;
		}

		var self = this;
		var content = {
			size: this.pageSize,
			index: this.pageIndex
		};
		if(this._resultuuid && !reload){
			content.resultuuid = this._resultuuid;
		}
		if(pageUrl){
			content.pageurl = pageUrl;
		}
		
		var url = com.ibm.relm.le.ui.queryServiceUri;
		var qinfo = self._queryInfo;
		var method = "POST"; //$NON-NLS-1$
		var headers = {"Accept":"application/json"}; //$NON-NLS-1$ //$NON-NLS-2$
		var hasBody = true;
		if(qinfo.sparql){
			this.showEditButton(true);
			content.sparql = qinfo.sparql;
		} else if(qinfo.query){
			content.fts = qinfo.query;
			content.scope = qinfo.scope || "all"; //$NON-NLS-1$
		} else if (qinfo.queryUri){
			this.showEditButton(true);
			method = "GET"; //$NON-NLS-1$
			url = qinfo.queryUri;
			headers = {"Accept":"application/sparql-results+json"}; //$NON-NLS-1$ //$NON-NLS-2$
		} else if (qinfo.resourceUri){
			content.sparql = "SELECT ?property ?value { <" + decodeURIComponent(qinfo.resourceUri) + "> ?property ?value }"; //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			return null;
		}

		var systemParameters = com.ibm.relm.common.web.ui.internal.queryparams.SystemParameterUtil.getSystemParameterValues();
		if(qinfo.parameterized === "true"/*We use a text string to represent the boolean value*/) { //$NON-NLS-1$
			if (!qinfo.parameterValues || (qinfo.parameterValues && self.repromptParameters && !self.previewMode)) {
				self.repromptParameters = false;
				self.promptForParameters(qinfo.parameterValues);
				return null;
			} else {
				qinfo.parameterValues = dojo.mixin(qinfo.parameterValues, systemParameters);
				content.parameterValues = dojo.toJson(qinfo.parameterValues);
				self.repromptParameters = false;
			}
		} else {
			content.parameterValues = dojo.toJson(systemParameters);
		}

		return {
			url: url,
			method: method,
			headers: headers,
			hasBody: hasBody,
			content: content
		};
	},
	
	_displayResources: function(relayout, columns){
		var info = {
			resources: this._resources,
			relayout: relayout,
			showIndex: true,
			showActions: false,
			columns: columns
		};
		if(this.pageSize > 0){
			info.startIndex = this.pageSize * this.pageIndex + 1;
		}
		this._displayList(info);
	},

	onResourceSelected: function(target){
		var self = this;
		var util = com.ibm.relm.le.ui.internal.util;

		var menu = new jazz.ui.Menu();
		var menuItem = new dijit.MenuItem({
			label: this._messages.OpenArtifact,
			iconClass: ICONCLASS_ARTIFACT,
			onClick: dojo.hitch(util, util.showArtifact, target._uri_)
		});
		menu.addChild(menuItem);
		if(!target._isPredicate_){
			menuItem = new dijit.MenuItem({
				label: this._messages.ShowProperties,
				iconClass: ICONCLASS_PROPERTIES,
				onClick: function(){
					var uri = 
						"#action=com.ibm.relm.explorer.web.showExplorer&resourceUri=" +  //$NON-NLS-1$
						encodeURIComponent(target._uri_);

					if(self.previewMode || self._queryInfo.isDirty || !self._queryInfo.queryUri){
						window.open(uri);
					} else {
						window.location.href = uri;
					}
				}
			});
			menu.addChild(menuItem);
		}
		var closeMenu = function(){
			dijit.popup.close(menu);
		};
		dijit.popup.open({
			parent: this, popup: menu, around: target,
			onExecute: closeMenu,
			onCancel: closeMenu,
			onClose: function() {
				menu.destroy();
			}
		});
		menu.focus();
		menu.connect(menu, "_onBlur", closeMenu); //$NON-NLS-1$
	},

	onResourceUpdated: function(){
		this._refreshContent();
	},

});
})();
