/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.le.ui.internal.util"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.le.ui.internal", "utilMessages"); //$NON-NLS-1$ //$NON-NLS-2$

// Trying to have loose coupling with product def modules, but this
// is better for users when trying to show a product .
try {
	jazz.core.loader.load_async('com.ibm.relm.ple.web.ui.productapi', function() { //$NON-NLS-1$
		// no op callback
	});
}
catch (e) {
	// Doesn't matter, this is optional. Not sure if this throws or not, but this doesn't hurt.
}


(function(){
	
// URIs
com.ibm.relm.le.ui.serviceRoot = net.jazz.ajax._contextRoot + "/"; //$NON-NLS-1$
com.ibm.relm.le.ui.leServiceUri = com.ibm.relm.le.ui.serviceRoot + "relmle/le/"; //$NON-NLS-1$
com.ibm.relm.le.ui.queryServiceUri = com.ibm.relm.le.ui.serviceRoot + "relmui/query/"; //$NON-NLS-1$
com.ibm.relm.le.ui.jrsServiceUri = com.ibm.relm.le.ui.serviceRoot + "relmui/sgw/"; //$NON-NLS-1$

var l = window.location;
var PRODUCT_URI_HEAD = l.protocol + '//' + l.host + net.jazz.ajax._contextRoot + '/productdef/resource/itemVersion_'; //$NON-NLS-2$ //$NON-NLS-1$
var PRODUCT_CONFIGURATION_URI_HEAD = l.protocol + '//' + l.host + net.jazz.ajax._contextRoot + '/productdef/resource/configurationVersion_'; //$NON-NLS-2$ //$NON-NLS-1$
l = undefined;

// Namespaces
var NS_AE = "http://relm.ibm.com/ae/"; //$NON-NLS-1$
com.ibm.relm.le.ui.namespace = {
	ae: {
		sparqlTemplate: NS_AE + "sparqlTemplate", //$NON-NLS-1$
		variable: NS_AE + "variable", //$NON-NLS-1$
		parameter: NS_AE + "parameter", //$NON-NLS-1$
		child: NS_AE + "child", //$NON-NLS-1$
		owner: NS_AE + "owner", //$NON-NLS-1$
		Folder: NS_AE + "Folder", //$NON-NLS-1$
		Ae: NS_AE + "ArtifactElement" //$NON-NLS-1$
	}
};

var _messages = dojo.i18n.getLocalization("com.ibm.relm.le.ui.internal", "utilMessages"); //$NON-NLS-1$ //$NON-NLS-2$

// utilities
com.ibm.relm.le.ui.internal.util = {
	getXhrErrorMessage: function(response, ioargs){
		var errorObject = {};
		var details = [];
		try {
			var contentType = ioargs.xhr.getResponseHeader("Content-Type"); //$NON-NLS-1$
			if(contentType.indexOf("text/json") !== 0 && contentType.indexOf("application/json") !== 0 && contentType.indexOf("application/javascript") !== 0){ //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				throw null;
			}
			errorObject = dojo.fromJson(ioargs.xhr.responseText || response.responseText);
			if (errorObject.explanation) {
				details.push({
					message: errorObject.explanation,
					code: jazz.ui.MessageBox.STATUS_TYPES.OK
				});
			}
			if (errorObject.userAction) {
				details.push({
					message: errorObject.userAction,
					code: jazz.ui.MessageBox.STATUS_TYPES.OK
				});
			}
			if(details.length === 0){
				if(!errorObject.summary) {
					if(errorObject.errorCause){
						errorObject.summary = errorObject.errorCause.errorMessage || errorObject.errorMessage || response.message;
					}
					if(!errorObject.summary) errorObject.summary = errorObject.errorMessage || response.message;
				}
				details.push({
					message: errorObject.errorCause ? (errorObject.errorCause.detailedMessage || errorObject.errorCause.errorMessage) : null,
					code: jazz.ui.MessageBox.STATUS_TYPES.ERROR
				});
			}
		}
		catch (e) {
			console.error('Unexpected error format. Not JSON.', e); //$NON-NLS-1$
			errorObject = {
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: _messages.unknownError
			};	
			details.push({
				message: ioargs.xhr.responseText || response.responseText,
				code: jazz.ui.MessageBox.STATUS_TYPES.ERROR
			});
		}
		return {
			type: errorObject.type || jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: errorObject.summary || errorObject || _messages.unknownError,
			details: details,
			closable: true
		};
	},

	/**
	 * Special way to show a product definition. If
	 * the productapi code is loaded (trying not to be too tightly coupled),
	 * this a special function is used that can directly open the product tree
	 * or show a disambiguration page if in multiple places. Otherwise, falls
	 * back to a simpler #action.
	 * @param  {string} uri URI to a product.
	 */
	showProduct: function(uri) {
		// This is a local Product Definition. Go directly to it in this window.
		// Note that if we were to load this OSLC style URL it would result in a redirect to the HTML
		// page, which is much slower since the browser treats it like a full page load, not an ajax load.
		// Just updating the #action is dramatically faster.
		if (dojo.exists('com.ibm.relm.ple.web.ui.productapi')) { //$NON-NLS-1$
			com.ibm.relm.ple.web.ui.productapi.findPathsToProductAndShow(uri);
		}
		else {
			// If 'productapi' is not loaded, could that mean that
			// all product actions are missing too? Is this a safe fallback?
			// Or should we just show the URL directly?
			window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
				action: 'ple.read_only_product', //$NON-NLS-1$
				product: uri
			});
		}
	},
	/**
	 * Check if specified uri is a product definition or product configuration.
	 * @param  {string} uri URI to artifact that may be a product or product configuration.
	 */
	isProduct: function(uri) {
		if (uri.indexOf(PRODUCT_URI_HEAD) === 0
			|| uri.indexOf(PRODUCT_CONFIGURATION_URI_HEAD) === 0) {
			return true;
		}
	},
	/**
	 * If the URI looks like it's too a local product definition,
	 * tries to efficiently show it in this window, directly in the product
	 * tree is possible.
	 * @param  {string} uri URI to artifact that may be a product.
	 * @return {boolean}     true if it was a product and was shown. Otherwise false.
	 */
	showIfProduct: function(uri) {
		if (this.isProduct(uri)) {
			this.showProduct(uri);
			return true;
		}
		else {
			return false;
		}
	},

	/**
	 * Escape HTML for RELM config.
	 * Text in RELM config is escaped in ViewConfigJSONUtil.java in 6.0.1, 
	 * but old config may contain "<" and ">" characters which should be escaped. 
	 */
	escapeHtml: function(text){
		return text.replace(/</g,"&lt;").replace(/>/g,"&gt;"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	unescapeHtml: function(text){
		return text.replace(/&lt;/g,"<").replace(/&gt;/g,">"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	/**
	 * Get sanitized uri.
	 */
	isValidUrl: function(url){
		// Make sure that the url will be considered valid by dojo Url class
		try {
			if(!dojo.isString(url)){
				return false;
			}
			var dojoUrl = new dojo._Url(url); // validate URL
			var scheme = (dojoUrl.scheme || "").toLowerCase();//$NON-NLS-1$
			if(scheme === "javascript" || scheme === "vbscript"){//$NON-NLS-1$//$NON-NLS-2$
				// don't allow protocols
				return false;
			}
		} catch (e) {
			return false;
		}
		return true;
	},
	
	sanitizeUrl: function(url) {
		if(com.ibm.relm.le.ui.internal.util.isValidUrl(url)){
			return url;
		}
		return "_" + url; // sanitize scheme part //$NON-NLS-1$
	},

	/**
	 * Shows an artifact (URI) in a new window, or this window
	 * it's a local product definition.
	 * 
	 * @param  {string} uri artifact URI.
	 * @param {boolean} openInNewWindow optional. If omitted or 'true',
	 * artifact will open a new window. If 'false', will reused this window.
	 */
	showArtifact: function(uri, openInNewWindow) {
		if (this.showIfProduct(uri)) {
			return;
		}
		else if(com.ibm.relm.le.ui.internal.util.isValidUrl(uri)){
			if (openInNewWindow === undefined || openInNewWindow) {
				var w = window.open(uri);
				w && w.focus();
			}
			else {
				// I think you can just use window.open with a target of _self, but haven't verified across browsers.
				dojo.doc.location = uri;
			}
		}
	},
	/**
	 * Get an option value in URL. e.g. https://server/relm?editSrc=true#...
	 * @param  {string} option name. e.g. editSrc
	 */
	getURLOption: function(option){
		var queryString = dojo.doc.location.search;
		// allow source editing
		if(queryString.length > 1){
			queryString = queryString.substring(1);
			return dojo.queryToObject(queryString)[option];
		}
		return null;
	},
	/**
	 * Check an option in URL. e.g. https://server/relm?editSrc=true#...
	 * @param  {string} option name. e.g. editSrc
	 */
	checkURLOption: function(option){
		var optvalue = this.getURLOption(option);
		return optvalue === "true";//$NON-NLS-1$
	}
};
})();
