/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.le.ui.internal.view.common.TableUtils"); //$NON-NLS-1$

(function(){
		
dojo.require("dijit.InlineEditBox"); //$NON-NLS-1$
dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$
dojo.require("dijit.form.FilteringSelect"); //$NON-NLS-1$
dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.view.common.Select"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.util"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.le.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var escapeHtml = com.ibm.relm.common.web.ui.util.escapeHtml;
var messages = dojo.i18n.getLocalization("com.ibm.relm.le.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	
com.ibm.relm.le.ui.internal.view.common.TableUtils = {};

var tableUtils = com.ibm.relm.le.ui.internal.view.common.TableUtils;
dojo.mixin(tableUtils, 
{
	createPropTableColumnsEx: function(nameChangeHandler, valueChangeHandler, typeChangeHandler, labelChangeHandler, descriptionChangeHandler, linkToLabelChangeHandler, linkToAEChangeHandler, scope){
		var nameOptions = new dojo.data.ItemFileReadStore({data: {identifier: "name", items:[]}}); //$NON-NLS-1$
		var linkToAEOptions = new dojo.data.ItemFileReadStore({data: {identifier: "value", items: []}}); //$NON-NLS-1$
		var columns = [
			{id: "name", name: messages.configEditorPropertyName, editable: true, options: nameOptions, callback: nameChangeHandler, preventEmpty: true, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
//			{id: "value", name: messages.configEditorParameterValue, editable: true, callback: valueChangeHandler, preventEmpty: false, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
//			{id: "type", name: messages.configEditorPropertyType, editable: true, callback: typeChangeHandler, preventEmpty: true, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
//			{id: "label", name: messages.configEditorPropertyLabel, editable: true, callback: labelChangeHandler, preventEmpty: false, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
//			{id: "description", name: messages.configEditorPropertyDescription, editable: true, callback: descriptionChangeHandler, preventEmpty: false, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
			{id: "linkToLabel", name: messages.configEditorPropertyLinkToLabel, editable: true, callback: linkToLabelChangeHandler, preventEmpty: true, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
			{id: "linkToAE", name: messages.configEditorPropertyLinkToAE, editable: true, options: linkToAEOptions, callback: linkToAEChangeHandler, preventEmpty: false, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
			{id: "actions", name: messages.configEditorAction, isAction: true} //$NON-NLS-1$
		];
		return tableUtils.createTableColumns(columns, scope || this);
	},

	/*
	 * columnDefArray: [ {id, name, editable, callback, preventEmpty, options, width, isAction}, ... ]
	 */
	createTableColumns: function(columnDefArray, scope){
		scope = scope || this;
		return dojo.map(columnDefArray, function(columnDef) { 
			return tableUtils.createTableColumnDescription(columnDef, scope); 
		}, scope);
	},
	
	createTableColumnDescription: function(columnDef, scope){
		var col = {
			id: columnDef.id,
			name: columnDef.name,
			isAction: columnDef.isAction
		};
		if(columnDef.editable){
			scope = scope || this;
			var self = tableUtils;
			var callback = dojo.hitch(scope, columnDef.callback);
			col.getCellContent = function(cellData) {
				var rowWidget = this;
				var cb = this.callback || callback;
				var editBox = self._createInlineEditBox(cellData, function(newValue) {
					// call user's call back 
					cb(rowWidget.data, col.id, newValue, rowWidget);
				}, columnDef.preventEmpty ? rowWidget.data[col.id]: null, columnDef.options, scope);
				return editBox;
			};
		}
		if(columnDef.width){
			col.style = {width: columnDef.width};
		}
		return col;
	},
	
	_getOption: function(options, value) {
		var option = null;
		if(dojo.isArray(options)){
			dojo.some(options, function(opt) {
				if (typeof opt.value == "boolean") { //$NON-NLS-1$
					if (opt.value == (value == true.toString())) {
						option = opt;
						return true;
					}
				} else if (opt.value == value) {
					option = opt;
					return true;
				}
			});
		}else if(options instanceof dojo.data.ItemFileReadStore){
			options.fetchItemByIdentity({identity: value, onItem: function(item, request){option = item;}});
		}
		return option;
	},
	
	_createInlineEditBox: function(cellData, onChangeCallback, valueForEmpty, options, scope){
		var title = cellData;
		if(options){
			if(dojo.isFunction(options)){
				options = options();
			}else if(options instanceof dojo.data.ItemFileReadStore){
			}else{
				// need copy to set selected 
				options = dojo.clone(options); 
			}
			
			var option = this._getOption(options, cellData);
			if (option) {
				option.selected = true;
				title = option.label;
			}
		}
		
		onChangeCallback = dojo.hitch(scope || this, onChangeCallback);
		
		var self = this;
		var noValueString = (valueForEmpty) ? "<span class='noValueIndicator'>" + messages.noValueIndicator + "</span>" : "_"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		var editor = null, editorParams = null;
		if(options){
			if(options instanceof dojo.data.ItemFileReadStore){
				editor = "dijit.form.ComboBox"; //$NON-NLS-1$
				editorParams = {
						maxHeight: 500,
						store: options,
						searchAttr: "label", //$NON-NLS-1$
						labelAttr: "label" //$NON-NLS-1$
				};
			}else{
				editor = "com.ibm.relm.le.ui.internal.view.common.Select"; //$NON-NLS-1$
				editorParams = {optionsArray : options};
			}
		}else{
			editor = "dijit.form.TextBox"; //$NON-NLS-1$
			editorParams = {optionsArray : options};
		}
		var editBox = new dijit.InlineEditBox({
			editorWrapper: InlineEditor,
			editor: editor,
			editorParams: editorParams,
			title : title,
			noValueIndicator: noValueString,
			onChange : function(value){
				if (!value || value.length === 0) {
					if(valueForEmpty){
						alert(messages.configEditorEmptyValue);
						this.set("value", valueForEmpty); //$NON-NLS-1$ 
						return;
					}else{
						value = "";//$NON-NLS-1$
					}
				}

				// replace value into label. 
				// com.ibm....view.common.Select widget shows value instead of label unexpectedly once option is selected.
				// Setting displayedValue in Select widget can resolve this label problem, but if you do so, InlineEditBox cannot distinguish
				// labels which have the same name. As a result, onChange event won't come up if you change option to one which has the same name as current one. 
				if (options) {
					var option = self._getOption(options, value);
					if (option) {
						var label = option.label;
						if (dojo.isArray(option.label)) {
							label = dojo.map(option.label, function(l) {return escapeHtml(l);});
						} else {
							label = escapeHtml(option.label);
						}
						this.domNode.innerHTML = label;
						if (this.displayNode) this.displayNode.title = option.label;
					}
				}
				onChangeCallback(value);
			}
		}, dojo.create("div", { //$NON-NLS-1$
			innerHTML : escapeHtml(title)
		}));
		return editBox;
	}
});
//override getValue of wrapperWidget of InlineEditBox to get item value of ComboBox when both of searchAttr and labelAttr are item label
var InlineEditor = dojo.declare("com.ibm.relm.le.ui.internal.view.common.TableUtils.InlineEditor", [dijit._InlineEditor], { //$NON-NLS-1$
	getValue: function(){
		var ew = this.editWidget;
		return String((ew.item && ew.item.value) || ew.get(("displayedValue" in ew || "_getDisplayedValueAttr" in ew) ? "displayedValue" : "value")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	}
});
})();