/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.le.ui.internal.view.edit.FolderSelectDialogContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.le.ui.internal", "FolderSelectDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
var ROOT_URI = "com.ibm.relm.le.ui.internal.view.edit.FolderSelectDialogContent.root"; //$NON-NLS-1$
var leServiceUtils = com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils;
var ns = com.ibm.relm.le.ui.namespace;

/**
 * Widget to Select a folder to save Artifact Element in
 */
dojo.declare("com.ibm.relm.le.ui.internal.view.edit.FolderSelectDialogContent", //$NON-NLS-1$
[ dijit._Widget, dijit._Templated],
{
	templatePath : dojo.moduleUrl("com.ibm.relm.le", "ui/internal/view/edit/templates/FolderSelectDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
	parentFolder: null,
	_folderMap: null,

	constructor: function(args){
		this.inherited(arguments);
		this._folderMap = {};
	},
	postMixInProperties: function(args) {
		this.inherited(arguments);
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.le.ui.internal", "FolderSelectDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._initFolderTree();
	},
	
	destroy: function(){
		this._clearFolderTree();
	},

	_initFolderTree: function() {
		this._clearFolderTree();

		this._addFoldersToTree();
	},
	_createFolderTree: function() {
		if(this._folderTreeWidget) {
			this._folderTreeWidget.destroy();
		}
		this._folderTreeWidget = new jazz.ui.Tree();
		// Hardcode this height to force scrollbars if there are too many
		// folders. This height is not a perfect match for the left side,
		// but works OK. Needed since we can't size relative to the table
		// cell we're in.
		this._folderTreeWidget.domNode.style.height = '81px'; //$NON-NLS-1$
		this._folderTree.appendChild(this._folderTreeWidget.domNode);

		//Create Root folder
		var item = this._createRootItem({uri: ROOT_URI, title: this.messages.rootElement});
		this._folderMap[ROOT_URI] = item;
		this._addFolder(null, item);
	},
	_clearFolderTree: function() {
		if(this._folderTreeWidget) {
			this._folderTreeWidget.destroy();
			this._folderTreeWidget = null;
		}
		this._folderMap  ={};
	},
	_addFoldersToTree: function() {
		this._createFolderTree();
		this._loadAllFolders();
	},
	_loadAllFolders: function(){
		var self = this;
		leServiceUtils.getElements().then(function(results){
			dojo.forEach(results, function(resource){
				if(resource.type === ns.ae.Folder){
					var item = self._createFolderItem(resource);
					self._folderMap[resource.uri] = item;
					self._addFolder(ROOT_URI, item);
					self._loadChildFolders(resource.uri);
				}
			});
		});
	},
	_loadChildFolders: function(parentUri){
		var self = this;
		leServiceUtils.getElements(parentUri).then(function(results){
			dojo.forEach(results, function(resource){
				switch(resource.type){
					case ns.ae.Folder:
						var item = self._createFolderItem(resource);
						self._folderMap[resource.uri] = item;
						self._addFolder(parentUri, item);
						self._loadChildFolders(resource.uri);
						break;
					case ns.ae.Ae:
						if(resource.uri === self.uri){
							self.parentFolder = parentUri;
						}
						break;
					default:
						break;
				}
				if(self.parentFolder){
					self._selectFolder(self.parentFolder);
				}
			});
		});
	},
	_addFolder: function(parentUri, child){
		var parent = this._folderMap[parentUri] || this._folderTreeWidget;
		parent.addChild(child);
	},
	_createRootItem: function(resource){
		return new jazz.ui.tree.IconItem({uri: resource.uri, label: com.ibm.relm.le.ui.internal.util.escapeHtml(resource.title), iconClass: "iconArtifactElement"}); //$NON-NLS-1$
	},
	_createFolderItem: function(resource){
		return new jazz.ui.tree.IconItem({uri: resource.uri, label: com.ibm.relm.le.ui.internal.util.escapeHtml(resource.title), iconClass: "iconFolder"}); //$NON-NLS-1$
	},
	_selectFolder: function(folderUri){
		if(!this._folderTreeWidget){
			return;
		}
		var folderItem = this._folderMap[folderUri];
		var parentItem = folderItem.getParent();
		while(parentItem){
			parentItem.setExpanded(true);
			parentItem = parentItem.getParent();
		}
		this._folderTreeWidget.setSelection(folderItem);
	},

	getValues: function() {
		if(!this._folderTreeWidget){
			return null;
		}
		var selection = this._folderTreeWidget.getSelection();
		var currentFolderNode = dojo.isArray(selection) && selection.length > 0 ? selection[0] : {};
		return currentFolderNode.uri === ROOT_URI ? null : currentFolderNode.uri;
	}
});
})();