/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'productapi_messages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var messages = dojo.mixin(
	dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'productapi_messages'), //$NON-NLS-1$ //$NON-NLS-2$
	dojo.i18n.getLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages')); //$NON-NLS-1$ //$NON-NLS-2$

var ICONS = {
	doorsBaseline: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/doorsBaseline.png'), //$NON-NLS-1$ //$NON-NLS-2$
	link: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/hyperlink_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$
	productImmutable: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/productInSnapshot_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$
	productMutable: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/product_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$
	productStream: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/product_stream.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	productStreamCheckedIn: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/product_stream_checked_in.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	productComponent: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/product_component.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	productComponentCheckedIn: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/product_component_checked_in.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	productConfiguration: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/configuration.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	blankGif: dojo.moduleUrl('dojo','resources/blank.gif'), //$NON-NLS-1$ //$NON-NLS-2$
	emptyPart: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/blank.gif'), //$NON-NLS-1$ //$NON-NLS-2$
	configVersion: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/product_configuration.gif') //$NON-NLS-1$ //$NON-NLS-2$
};

var htmlEscape = com.ibm.relm.common.web.ui.util.escapeHtml;

/**
 * Functions to format labels for products, that is, icons and text decorators to show their
 * state, snapshot, and branch.
 */
dojo.mixin(dojo.getObject('com.ibm.relm.ple.web.ui.formatting'), { //$NON-NLS-1$


	/**
	 * Formats a creator's username as HTML. This mainly catches the case where there
	 * is no user name (empty string or undefined), which happens to products that are
	 * implicitly checked out to modify a child product.
	 * @param  {string} creatorUserName username of creator, or empty string or undefined if none.
	 * @return {string} HTML fragment for the username. Safe to insert with innerHTML (already escaped).
	 */
	formatCreator: function(creatorUserName) {
		if (creatorUserName) {
			return htmlEscape(creatorUserName);
		}
		else {
			return '<span class="ple-noCreator" title="' + htmlEscape(messages.noCreatorToooltip) + '">' + htmlEscape(messages.noCreatorName) + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
	},

	/**
	 * Formats an indirect creator's username as HTML. This behaves similarly to formatCreator(), but
	 * formats the creator name differently because the creator didn't explicitly create the item.
	 * @param  {string} indirectCreatorUserName username of indirect creator, or empty string or undefined if none.
	 * @return {string} HTML fragment for the username. Safe to insert with innerHTML (already escaped).
	 */
	formatIndirectCreator: function(indirectCreatorUserName) {
		if (indirectCreatorUserName) {
			return '<span class="ple-noCreator" title="' + htmlEscape(messages.indirectCreatorTooltip) + '">' + htmlEscape(indirectCreatorUserName) + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		else {
			return '<span class="ple-noCreator" title="' + htmlEscape(messages.noCreatorTooltip) + '">' + htmlEscape(messages.noCreatorName) + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
	},

	getIconClassAndAltTextForItem: function(item){
		if (!item || !item.type) {
			return 'ple-icon-productInSnapshot'; //$NON-NLS-1$
		}
		var type = item.type;

		// Cannot add class for disabled hierarchy here since the artifacts with icons don't come through this
		if (type === 'link') //$NON-NLS-1$
		{
			return ['ple-icon-artifactLink', messages.artifactAlt]; //$NON-NLS-1$
		}
		else if (type === 'configVersion') { //$NON-NLS-1$
			if (item.isImmutable) {
				return ['ple-icon-configVersionBaseline', messages.configVersionBaselineAlt]; //$NON-NLS-1$
			}
			else if (item.isPatch) {
				return ['ple-icon-configVersionPatch', messages.configVersionPatchAlt]; //$NON-NLS-1$
			}
			else {
				return ['ple-icon-configVersion', messages.configVersionAlt]; //$NON-NLS-1$
			}
		}
		else if (type === 'emptyPart') { //$NON-NLS-1$
			return ['ple-icon-emptyPart', messages.emptyPartAlt]; //$NON-NLS-1$
		}
		else if (type === 'product') { //$NON-NLS-1$
			if (item.rtcStream) {
				if (item.isImmutable){ //$NON-NLS-1$
					return ['ple-icon-productStreamCheckedIn', messages.stream]; //$NON-NLS-1$
				}
				else {
					return ['ple-icon-productStream', messages.stream]; //$NON-NLS-1$
				}
			}
			if (item.rtcComponent) {
				if (item.isImmutable){ //$NON-NLS-1$
					return ['ple-icon-productComponentCheckedIn', messages.component]; //$NON-NLS-1$
				}
				else {
					return ['ple-icon-productComponent', messages.component]; //$NON-NLS-1$
				}
			}

			if (item.conditionVariables !== undefined) {
				return ['ple-icon-rtcConfiguration', messages.configuration]; //$NON-NLS-1$
			}

			var immutable = item.isImmutable; //$NON-NLS-1$

			return immutable ?
				['ple-icon-productInSnapshot', messages.productInSnapshotAlt] : //$NON-NLS-1$
				['ple-icon-product', messages.productAlt]; //$NON-NLS-1$
		}
		else {
			throw new Error('getIconClassAndAltTextForItem - Unknown type: ' + type); //$NON-NLS-1$
		}
	},

	getIconClassForItem: function(item){
		return this.getIconClassAndAltTextForItem(item)[0];
	},

	formatProductIconForItem: function(item) {
		var classAndAlt = this.getIconClassAndAltTextForItem(item);
		return '<img class="' + classAndAlt[0] + '" alt="' + classAndAlt[1] + '" src="' + ICONS.blankGif + '">'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	},

	/**
	 * Returns HTML for a product or artifact that include its leading icon, title, and decorator text.
	 * 
	 * @param  {object} item            the JSON item to display.
	 * @param  {boolean} wrapTitleInSpan if true, an extra span.ple-title is
	 *     wrapped around the title text itself. This is normally not needed,
	 *     but is used by the Find Use page to hide and show titles (i.e.,
	 *     just show some resources as icons).
	 * @return {string}                 HTML display of the item.
	 */
	formatProductIconAndTitleForItem: function(item, wrapTitleInSpan) {
		if (item.type == "link") { //$NON-NLS-1$
			// We'll prefer icons for links still since OSLC previews return icon src links
			// for new icons. Once those are applied we'd need to remove the ple-icon-artifactLink class,
			// but I haven't tracked down all the places to do that. jazz.ui.ResourceLink does not do that for us.
			var imgSrc = item.isDoorsBaseline ? ICONS.doorsBaseline : ICONS.link;
			return '<img class="ple-icon" alt="' + messages.artifactAlt + '" src="' + imgSrc + '">' + this.formatProductTitleForItem(item); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		else {
			var classAndAlt = this.getIconClassAndAltTextForItem(item);
			var title = this.formatProductTitleForItem(item);

			return wrapTitleInSpan
				? '<img class="ple-icon ' + classAndAlt[0] + '" alt="' + classAndAlt[1] + '" src="' + ICONS.blankGif + '"> <span class="ple-title">' + title + '</span>' //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
				: '<img class="ple-icon ' + classAndAlt[0] + '" alt="' + classAndAlt[1] + '" src="' + ICONS.blankGif + '">' + title ; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
	},

	/**
	 * Get HTML string for an IMG of an Item, not a particular version.
	 * 
	 * @param  {object} item an item version to format as an item.
	 * @return {[type]}      [description]
	 */
	formatProductItemIconAndTitleForItem: function(item) {
		return '<img class="ple-icon ple-icon-product" alt="' +  messages.productAlt + '" src="' + ICONS.blankGif + '">' + this.formatProductItemTitleForItem(item); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},

	/**
	*	This is meant for use by the product audit log only. The icon it uses is always the default one for a mutable product. 
	*/
	formatDefaultProductIconAndTitle: function(type, name, branchName, snapshot, immutable) {
		var imgSrc = ICONS.productMutable;
		return '<img class="ple-icon" src="' + imgSrc + '" alt="" title="' + messages.productAlt + '">' + this.formatProductTitle(name, branchName, snapshot); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},

	/**
	 * Given a dimension, creates and icon and title for it indicating if it's
	 * local or shared. If local and not for a config version, it is labeled
	 * "Local" (this the current product). If it is local, but on a config
	 * version, then the product's item name is used as the scope (name of the
	 * product), but the icon is the same either way.
	 * 
	 * @param  {object}  dimension       JSON dimension with expected details.
	 * @param  {Boolean} isConfigVersion true if formatting to show on a
	 *     config version, otherwise false. Will change display of local
	 *     dimensions.
	 * @return {HTML string}                  the scope label.
	 */
	formatDimensionScope: function(dimension, isConfigVersion) {
		if (dimension.shared) {
			return '<span class="j-label">' //$NON-NLS-1$
				+ '<img class="ple-icon-dimensionShared ple-icon" src="' + ICONS.blankGif + '" title="' + messages.sharedDimensionTooltip + '">' //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				+ htmlEscape(messages.sharedDimensionScope)
				+ '</span>'; //$NON-NLS-1$
		}
		else {
			var name = isConfigVersion ? dimension.itemTitle : messages.localDimensionScope;

			return '<span class="j-label">' //$NON-NLS-1$
				+ '<img class="ple-icon ple-icon-dimensionProductLocal" alt="" title="' +  messages.localDimensionScopeTooltip + '" src="' + ICONS.blankGif + '">' //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				+ htmlEscape(name)
				+ '</span>'; //$NON-NLS-1$
		}
	},

	formatDecoratorForItem: function(item) {
		var args = {
			snapshot: htmlEscape(item._snapshot),
			branch: htmlEscape(item._branch_name)
		};

		if (item._snapshot && item.branchName) {
			return dojo.replace(messages.branchXSnapshotX_html, args);	
		}
		else if (item._snapshot) {
			return dojo.replace(messages.snapshotX_html, args);		
		}
		else if (item.branchName) {
			return dojo.replace(messages.branchX_html, args);
		}
		else {
			return ''; //$NON-NLS-1$
		}
	},
	
	formatSnapshotTitle: function(snapshot) {
		var args = {
				snapshot: snapshot._snapshot || '' //$NON-NLS-1$
		};
		return dojo.replace(messages.snapshotTitleX_html, args);
	},
	
	formatBranchTitle: function(branch) {
		var args = {
				branch: branch.branchName
		};		
		if (!args.branch) {
			args.label = messages.noDimensionsLabel;
			return dojo.replace(messages.grayLabel_html, args);
		}
		else{
			return dojo.replace(messages.branchTitleX_html, args);
		}
	},

	_formatProductTitle: function(params) {
		var template;

		if (params.branch && params.snapshot) {
			template = params.rtcBaselineName
				? messages.productTitleXRtcBaselineXBranchXSnapshotX_html
				: messages.productTitleXBranchXSnapshotX_html;
		}
		else if (params.branch) {
			template = params.rtcBaselineName
				? messages.productTitleXRtcBaselineXBranchX_html
				: messages.productTitleXBranchX_html;
		}
		else if (params.snapshot) {
			template = params.rtcBaselineName
				? messages.productTitleXRtcBaselineXSnapshotX_html
				: messages.productTitleXSnapshotX_html;
		}
		else {
			template = messages.productTitleX_html;
		}

		if (!params.title) {
			// Strip leading snapshot dash if no title.
			template = template.replace(/^ - /, ''); //$NON-NLS-1$
		}

		var title = dojo.replace(template, function(wholeMatch, name) {
			return htmlEscape(params[name]);
		});

		return title;

	},

	formatProductTitle: function(title, branch, snapshot) {
		var params = {
			title: title,
			branch: branch,
			snapshot: snapshot
		};

		return this._formatProductTitle(params);
	},

	formatProductItemTitleForItem: function(item) {
		return this._formatProductTitle({
			title: item.name
		});
	},

	formatProductTitleForItem: function(item) {

		return this._formatProductTitle({
			title: item.name,
			branch: item.branchName,
			
			// Hack the 5.0 version or baseline name into the snapshot name for now.
			snapshot: (item.type == "configVersion") ? item.baselineName : item.version, //$NON-NLS-1$
			rtcBaselineName: item.rtcBaselineName
		});
	},

	/**
	 * Returns a new function that can be used with array.sort to sort product
	 * items by their labels (including branch and version). This returns a
	 * new function every time since the function will cache the formatted
	 * displayname, which could take some time to compute for large lists of
	 * products.
	 * 
	 * @return {function} new compare function to sort products with.
	 */
	newProductItemCompareFunction: function() {
		var titleCache = {};
		
		var self = this; 
		
		function getTitle(product) {
			var uri = product.uri;
			var title = titleCache[uri];
			if (!title) {
				title = self.formatProductTitleForItem(product);
				titleCache[uri] = title;
			}
			return title;
		}

		return function sortProduct(left, right) {
			left = getTitle(left);
			right = getTitle(right);

			if (left < right) {
				return -1;
			}
			else if (left === right) {
				return 0;
			}
			else {
				return 1;
			}
		};
	}
});

})();
