/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.ui.pages.RemoveDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "RemoveDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function(){
	var Site = com.ibm.relm.ple.web.ui.Site;
	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'RemoveDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
	var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
	var contextRoot = net.jazz.ajax._contextRoot;

		dojo.declare(
		"com.ibm.relm.ple.web.ui.pages.RemoveDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/RemoveDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			dialogTitle: messages.removeDialogTitle,
			prompt: "", //$NON-NLS-1$
			details: "", //$NON-NLS-1$
			detailsList: "", //$NON-NLS-1$
			width: '500px', //$NON-NLS-1$

			/**
			 * If removing child products, then they all must be under the same parent
			 * and all have this same context config version uri. If removing
			 * top-level config versions, this can be null.
			 * @type {string}
			 */
			contextConfigVersionUri: null,

			/**
			 * All the child item versions, config version, and empty parts to be removed or deleted.
			 * 
			 * If deleting top level products the 'part' field can be null (in that case there aren't
			 * 'child' products really, but the same name is used). Can be empty.
			 * @type {array}
			 */
			removeSites: [],

			canDelete: true,

			constructor: function(args) {
				this.messages = messages;
				this.iconSrc = dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/question.gif'); //$NON-NLS-1$ //$NON-NLS-2$
				this.name = args.name;

				var linkTargets = dojo.filter(args.removeSites, function(site) {
					return !!site.link;
				});

				this._linkCount = linkTargets.length;
			},

			postMixInProperties: function() {
				this.inherited(arguments);

				var targetLength = this.removeSites.length;

				if (!this.contextConfigVersionUri && this._linkCount === 0) {
					// No parent. Only top level config versions.
					this.prompt = targetLength == 1 ?
						dojo.replace(messages.deleteXPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(this.name)]) :
						dojo.replace(messages.deleteXConfigurationsPrompt, [targetLength]);

					this.details = messages.configurationWillBeDeletedForever;
					this.detailsList = messages.deleteConfigurationDetailsList;
				}
				else if (this._linkCount === targetLength) {
					// Only links
					this.prompt = this._linkCount > 1 ?
						dojo.replace(messages.removeXLinksPrompt, [this._linkCount]) :
						dojo.replace(messages.removeXPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(this.name)]);

					this.details = messages.removeLinkDetails;
				}
				else {
					// Mix of possibly config versions, item versions, empty parts and links
					if (targetLength > 1) {
						this.prompt = dojo.replace(messages.removeXItems, [targetLength]);
						this.details = messages.removeItemsDetails;
					}
					else {
						// a single product or configuration that is modifiable
						if (this.canDelete) {
							this.prompt = dojo.replace(messages.deleteXPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(this.name)]);
							if (this.removeSites[0].type == "product") { //$NON-NLS-1$
								this.details = messages.deleteProductDetails;
							}
							else if (this.removeSites[0].type == "configVersion") { //$NON-NLS-1$
								this.details = messages.deleteConfigDetails;								
							}
							else {
								this.details = messages.deleteOption;
							}
						}
						else {
							this.prompt = dojo.replace(messages.removeXPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(this.name)]);
							this.details = messages.removeItemDetails;
						}
					}
				}
			},

			cancel: function() {
				this._dialog.close();
			},

			postCreate: function() {
				this.inherited(arguments);

				//If contextConfigVersionUri is undefined, then that means we have a
				//top level product and the dialog displayed will be different
				if (this.contextConfigVersionUri || this._linkCount > 0) {
					if (this.canDelete) {
						this.removeButton = this.addButton(
							this.messages.deleteProductButton,
							dojo.hitch(this, this.onRemove, true),
							false);
					}
					else {
						this.removeButton = this.addButton(
							this.messages.removeProductButton,
							dojo.hitch(this, this.onRemove, false),
							false);
					}
				}
				else {
					this.deleteButton = this.addButton(
						this.messages.deleteProductButton,
						dojo.hitch(this, this.onRemove, true),
						true);
				}

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.canDelete ? this.messages.deleteDialogTitle : this.dialogTitle ,
					width: this.width,
					destroyContents: true,
					contentNode: this.domNode,
					closable: false,
					iconClass: this.canDelete ? 'ple-icon-delete' : 'ple-icon-remove' //$NON-NLS-1$ //$NON-NLS-2$
				});
			},

			addButton: function(title, callback, isDefaultButton) {
				var button = dojo.create('button'); //$NON-NLS-1$
				button.appendChild(document.createTextNode(title));

				this.connect(button, 'onclick', callback); //$NON-NLS-1$

				if (isDefaultButton) {
					this._defaultButton = button;
				}

				dojo.place(button, this._customButtonsNode, 'last'); //$NON-NLS-1$

				//Don't know if this is safe memory leak wise
				return button;
			},

			onRemove: function(isDelete) {
				if (this.removeButton) this.removeButton.disabled = true;
				if (this.deleteButton) this.deleteButton.disabled = true;
				this.cancelButton.disabled = true;

				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.removingStatus);

				var content = {
					sites: Site.sitesToJsonSites(this.removeSites),
					andDelete: isDelete,
					// product configurations need to be removed in all delete cases as well
					andRemove: !this.contextConfigVersionUri || this.removeSites[0].type == 'configVersion' ? true : !isDelete //$NON-NLS-1$
				};

				var self = this;
				jazz.client.xhrPost({
					url: contextRoot + '/ple/remove_anything', //$NON-NLS-1$,
					content: content,
					load: function() {
						dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$

						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							clearPath: true
						}).addBoth(function() {
							self._dialog.close();
						});
					},
					error: function(error) {
						if (self.removeButton) self.removeButton.disabled = false;
						if (self.deleteButton) self.deleteButton.disabled = false;
						self.cancelButton.disabled = false;

						dojo.publish('ple.refresh'); //$NON-NLS-1$

						dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$
						dojo.publish('ple.product_tree_xhrError', [error, true]); //$NON-NLS-1$

						// TODO: show error in THIS dialog.
						self._dialog.close();
					},
					handle: function() {
						message.hide();
					}
				});
			},

			startup: function() {
				if (this._defaultButton) {
					var self = this;
					this.connect(window.document, 'onkeydown', function(e) { //$NON-NLS-1$
						if (e.keyCode == dojo.keys.ENTER) {
							self._defaultButton.click();
						}
					});
				}
			}

		});

	})();